#include <string>
#include <iostream>
#include <vector>

#include "schema.h"
#include "iopclib/cache/voidCache.h"

using namespace std;
using namespace iopc;

void setupDb(Connection* conn)
{
	// drop script
	// script execution can throw exceptions as the dropped structures may not be present in the database
	vector<string> script;
	script = ScriptsGenerator::getDbDropScript(conn->getDriver());
	for(vector<string>::const_iterator it = script.begin(); it != script.end(); it++) {
		try {
			conn->sqlNonQuery(*it);
		} catch (IopcException& e) {
			cout << e.what() << endl;
		}
	}

	// create script
	// If the script fails, some of the tables couldn't be dropped and recreated because of dependencies
	// created by other examples.
	// You have to resolve the dependencies and drop these tables from SQL*Plus manually:
	// select name, referenced_name from user_dependencies where referenced_name='objectname';
	script = ScriptsGenerator::getDbCreateScript(conn->getDriver());
	for(vector<string>::const_iterator it = script.begin();
		it != script.end(); it++) {
		conn->sqlNonQuery(*it);
	}

	// Prepare the blacklist table
	conn->sqlNonQuery("DROP TABLE blacklist");
	conn->sqlNonQuery("CREATE TABLE blacklist(id NUMBER PRIMARY KEY, name VARCHAR2(20))");
	conn->sqlNonQuery("INSERT INTO blacklist VALUES(1, 'Richard Doe')");
	conn->sqlNonQuery("INSERT INTO blacklist VALUES(2, 'Ola Nordmann')");

}

int main(int argc, char *argv[])
{
	CachedDatabase* db = NULL;
	CachedConnection* conn = NULL;
	try {
		// Uncomment the INFO_SQL mask to see what SQL statements are sent to the database
		Iopc::init(LogWriter::ERROR | LogWriter::WARNING /*| LogWriter::INFO_SQL*/, LogWriter::NONE);

		// Create a cached connection
		db = new CachedDatabase(
				DriverManager::getDriver("IopcOracle10g").getDatabase("XE"),
				// We use the basic cache implementation - the VoidCache
				new VoidCache()
			);
		conn = (CachedConnection*) db->getConnection("username=iopc;password=iopc");
		conn->open();

		// Create the database schema
		setupDb(conn);

		// Insert some objects
		DbPtr<Person> person;
		person->name = "Mary Major";
		person->age = 60;
		person.bePersistent(conn);

		DbPtr<Employee> employee;
		employee->name = "Ola Nordmann";
		employee->age = 45;
		employee->salary = 60000;
		employee.bePersistent(conn);

		DbPtr<Student> student;
		student->name = "Richard Doe";
		student->age = 22;
		student->studcardid = "WCD-3223";
		student->supervisor = employee;
		student.bePersistent(conn);

		DbPtr<PhdStudent> phd;
		phd->name = "Joe Bloggs";
		phd->age = 27;
		phd->studcardid = "PHD-1234";
		phd->scholarship = 12000;
		phd->supervisor = employee;
		phd.bePersistent(conn);
		conn->commit();

		// CachePointer - lock the object in the cache
		CachePtr<PhdStudent> phdPtr = phd.getCachePtr();
		// or by dereferencing the DbPtr using the * operator:
		//CachePtr<PhdStudent> phdPtr = *phd;

		// Update the PhdStudent
		phdPtr->scholarship = 13000;
		phdPtr->supervisor = employee;
		phdPtr.release();

		// List persons older than 30
		SimpleQuery q("$::Person::age$ >= 30");
		cout << endl << "Persons older than 30" << endl;
		Result<Person> r(conn, q);
		r.open();
		for (Result<Person>::iterator it = r.begin(); it != r.end(); it++) {
			cout << "name: " << it->name << " age: " << it->age << endl;
		}
		r.close();

		// Find PhdStudents supervised by employee
		// This time we will select only from the simple view
		FreeQuery q2("$::PhdStudent[db.view.sv]$ WHERE $::PhdStudent[db.view.sv]::supervisor$ = " + employee->oid.toString());
		cout << endl << "Phd students supervised by " << employee->name << endl;
		Result<PhdStudent> r2(conn, q2);
		r2.open();
		for (Result<PhdStudent>::iterator it = r2.begin(); it != r2.end(); it++) {
			cout << "name: " << it->name << " age: " << it->age << " scholarship: " << it->scholarship << endl;
		}
		r2.close();

		// Database objects

		// Find a black list entry with externalId = 2
		KeyValues key;
		key.add("externalId", 2);
		DbPtr<BlacklistEntry> entry(conn, key);
		cout << endl << "Blacklist entry 2: " << endl;
		cout << entry->externalId << ": " << entry->name << endl;


		// Transient objects as query results
		cout << endl << "All tables from the USER_TABLES dictionary view" << endl;
		Result<UserTable> userTables(conn);
		userTables.open();
		for (Result<UserTable>::iterator it = userTables.begin(); it != userTables.end(); it++) {
			cout << it->tableName << " in " << it->tablespaceName << endl;
		}
		userTables.close();

		cout << endl << "Tables from the USERS tablespace ordered by name" << endl;
		SimpleQuery q3("$::UserTable::tablespaceName$ = 'USERS'", "$::UserTable::tableName$ ASC");
		Result<UserTable> userTables2(conn, q3);
		userTables2.open();
		for (Result<UserTable>::iterator it = userTables2.begin(); it != userTables2.end(); it++) {
			cout << it->tableName << " in " << it->tablespaceName << endl;
		}
		userTables2.close();

		cout << endl << "Tables with column counts" << endl;
		FreeQuery q4("USER_TABLES JOIN USER_TAB_COLS USING (TABLE_NAME) GROUP BY TABLESPACE_NAME, TABLE_NAME");
		Result<UserTable2> userTables3(conn, q4);
		userTables3.open();
		for (Result<UserTable2>::iterator it = userTables3.begin(); it != userTables3.end(); it++) {
			cout << it->tableName << " " << it->tablespaceName << " " << it->columnCount << endl;
		}
		userTables3.close();


		conn->close();
		db->returnConnection(conn);
		DriverManager::getDriver("IopcOracle10g").returnDatabase(db);

		conn = NULL;
		db = NULL;

		Iopc::shutdown();
	} catch (IopcException& e) {
		cout << e.what() << endl;
	}

	if (db && conn) conn->close();
	if (db) DriverManager::returnDatabase(db);

	cout << "-----------" << endl;
	cout << "Press enter" << endl;
	cin.getline(NULL, 0);
	return 0;
}


