/**
 * @file exceptionBase.h
 *
 * @author Jan Hadrava
 * @author Petr Cermak
 *
 * @brief Basic library exceptions
 *
 * @ingroup iopccommon
 */

#ifndef IOPCEXCEPTIONBASE_H
#define IOPCEXCEPTIONBASE_H

#include "defs.h"
#include <exception>
#include <string>

namespace iopc {

/**
 * @brief Base class of all IOPC 2 exceptions
 *
 * @ingroup iopccommon
 */
class DLL_EXTERNAL IopcException: public std::exception {
public:
	IopcException(const std::string& msg)
		throw ();
	virtual const char* what() const
		throw();
	virtual ~IopcException()
		throw();
public:
	static const std::string name;
	std::string message;

protected:
	IopcException(const std::string& exceptionName, const std::string& msg)
		throw();
};


/**
 * @brief File input/output error
 *
 * @ingroup iopccommon
 */
class DLL_EXTERNAL IopcIOException : public IopcException {
public:
	IopcIOException(const std::string& msg = "")
		throw ();
public:
	static const std::string name;
};

/**
 * @brief Not supported operation requested
 *
 * @ingroup iopccommon
 */
class DLL_EXTERNAL IopcNotSupportedException : public IopcException {
public:
	IopcNotSupportedException(const std::string& msg = "")
		throw ();
public:
	static const std::string name;
};


/**
 * @brief No free memory left
 *
 * Thrown mostly when new or malloc failed.
 * Do not pass any parameter as allocation of concatenated std::string message may fail.
 *
 * @ingroup iopccommon
 */
class DLL_EXTERNAL IopcNoMemoryException : public IopcException {
public:
	IopcNoMemoryException(const std::string& msg = "")
		throw ();
public:
	static const std::string name;
};

/**
 * @brief Incorrect usage
 *
 * @ingroup iopccommon
 */
class DLL_EXTERNAL IopcIncorrectUsageException : public IopcException {
public:
	IopcIncorrectUsageException(const std::string& msg = "")
		throw ();
public:
	static const std::string name;
};

/**
 * @brief Object/component is in an invalid state
 *
 * @ingroup iopccommon
 */
class DLL_EXTERNAL IopcInvalidStateException : public IopcException {
public:
	IopcInvalidStateException(const std::string& msg = "")
		throw ();
public:
	static const std::string name;
};

/**
 * @brief Deadlock in thread synchronization
 *
 * @ingroup iopccommon
 */
class DLL_EXTERNAL IopcDeadlockException : public IopcException {
public:
	IopcDeadlockException(const std::string& msg = "")
		throw ();
public:
	static const std::string name;
};

/**
 * @brief Lack of system resources or privileges
 *
 * @ingroup iopccommon
 */
class DLL_EXTERNAL IopcRsrcLackException : public IopcException {
public:
	IopcRsrcLackException(const std::string& msg = "")
		throw ();
public:
	static const std::string name;
};


/**
 * @brief Database level exception
 *
 * @ingroup iopccommon
 */
class DLL_EXTERNAL IopcDbException : public IopcException {
public:
	IopcDbException(const std::string& msg = "")
		throw ();

	static const std::string name;

protected:
	IopcDbException(const std::string& exceptionName, const std::string& msg)
		throw();
};


/**
 * @brief Deadlock in DB
 *
 * This exception can be solved by rollbacking and re-executing current transaction
 *
 * @ingroup iopccommon
 */
class DLL_EXTERNAL IopcDbDeadlockException : public IopcDbException {
public:
	IopcDbDeadlockException(const std::string& msg = "")
		throw ();

	static const std::string name;
};

/**
 * @brief Attempt to insert duplicate key into a collection
 *
 * @ingroup iopccommon
 */
class DLL_EXTERNAL IopcDuplicateKeyException : public IopcException {
public:
	IopcDuplicateKeyException(const std::string& msg = "")
		throw ();

	static const std::string name;
};

/**
 * @brief Key not found in a collection
 *
 * @ingroup iopccommon
 */
class DLL_EXTERNAL IopcNotFoundException : public IopcException {
public:
	IopcNotFoundException(const std::string& msg = "")
		throw ();

	static const std::string name;
};

/**
 * @brief Wrong metadata specified
 *
 * @ingroup iopccommon
 */
class DLL_EXTERNAL IopcMetadataException : public IopcException {
public:

	/**
	 * @param metadata A metadata code
	 * @param location Class name/attribute name or other location
	 * @param msg Error message
	 */
	IopcMetadataException(const std::string& metadata, const std::string& location, const std::string& msg)
		throw ();

	static const std::string name;
};

/**
 * @brief A row was not fetched although it was expected
 *
 * @ingroup iopccommon
 */
class DLL_EXTERNAL IopcNoRowsFetchedException : public IopcException {
public:
	IopcNoRowsFetchedException(const std::string& msg = "")
		throw ();

	static const std::string name;
};


}

#endif
