/**
 * @file cursorImpl.h
 *
 * @author Petr Cermak
 *
 * @ingroup iopcdb
 */

#ifndef CURSORIMPL_H_
#define CURSORIMPL_H_

#include "iopccommon/defs.h"
#include "iopcmeta/type.h"
#include "iopccommon/metadataHolder.h"

namespace iopc {


/**
 * @brief Interface of driver-side part of the @a Cursor class implementation
 *
 * Each database driver must provide a class that implements this interface
 *
 * Instances of this class are obtained using the @a ConnectionImpl::getCursor method
 * and are returned using the @a ConnectionImpl::returnCursor() method.
 *
 * CursorImpl implementation is responsible for SQL statement execution, parameter binding
 * and query results retrieval.
 *
 * @a Cursor executes operations of this interface in the following order:
 * <ol>
 * 	<li>@a CursorImpl::open()</li>
 *  <li>@a CursorImpl::prepare()</li>
 *  <li>@a CursorImpl::preExecBind()</li>
 *  <li>... binds as many IN parameters as required</li>
 *  <li>@a CursorImpl::execute()</li>
 *  <li>@a CursorImpl::preFetchBind()</li>
 *  <li>... binds as many OUT parameters as required</li>
 *  <li>Optionally continues with step 2 without binding the parameters</li>
 *  <li>@a CursorImpl::close()</li>
 * </ol>
 *
 * @ingroup iopcdb
 */
class  DLL_EXTERNAL CursorImpl {

public:
	/**
	 * @brief Opens the cursor
	 *
	 * Implementation should allocate all resources needed for the SQL statement execution
	 */
	virtual void open() = 0;

	/**
	 * @brief Closes the cursor
	 *
	 * Implementation should release all resources allocated in the @a CursorImpl::open() method
	 */
	virtual void close() = 0;

	/**
	 * @brief Prepares the SQL statement for execution
	 */
	virtual void prepare() = 0;

	/**
	 * @brief Binds an input parameter to a place in the prepared statement
	 *
	 * @param pos Position of the parameter in the prepared statement
	 * @param varAddr Address of the parameter value to be bound
	 * @param size Memory size of the parameter in bytes
	 * @param varType Data type of the parameter. @a varType refers only to basic datatypes (IOPC_TYPECLASS_SIMPLE)
	 *   or to string datatypes (IOPC_TYPECLASS_STRING). If the IOPC_TYPECLASS_STRING type is specified, the
	 *   varAddr refers to standard zero-terminaded C string representation.
	 * @param setNull Indicates, that instead the parameter should be a database NULL value. In this case, the varAddr pointer may be NULL.
	 * @param attributes Additional driver dependent parameters.
	 */
	virtual void preExecBind(const int pos, const void* varAddr, unsigned int size, const Type& varType, bool setNull, const MetadataHolder& attributes = MetadataHolder::emptyMeta) = 0;

	/**
	 * @brief Executes the statement
	 */
	virtual void execute() = 0;

	/**
	 * @brief Binds an output parameter to a place in the result set
	 *
	 * @param pos Position of the parameter in the result set
	 * @param varAddr Address of the output buffer where the fetched value will be stored.
	 * @param size Buffer size allocated to hold the fetched value in bytes
	 * @param varType Data type of the parameter. @a varType refers only to basic datatypes (IOPC_TYPECLASS_SIMPLE)
	 *   or to string datatypes (IOPC_TYPECLASS_STRING).
	 * @param isNull Pointer to an indicator variable which will be set to true if the fetched value is database NULL.
	 * 	 A NULL pointer may be passed to ignore the database NULL value notifiaction.
	 * @param attributes Additional driver dependent parameters.
	 */
	virtual void preFetchBind(const int pos, const void* varAddr, unsigned int size, const Type& varType, bool* isNull = NULL, const MetadataHolder& attributes = MetadataHolder::emptyMeta) = 0;

	/**
	 * @brief Fetches next result from the result set.
	 *
	 * Output parameters and NULL indicators specified in @a CursorImpl::preFetchBind are updated.
	 */
	virtual bool fetchNext() = 0;

	/**
	 * @brief Fetches previous result from the result set.
	 *
	 * Output parameters and NULL indicators specified in @a CursorImpl::preFetchBind are updated.
	 *
	 * @throws IopcNotSupportedException if this operation is not supported by the database driver.
	 */
	virtual bool fetchPrev() = 0;

	/**
	 * @brief Returns current position in the result set
	 */
	virtual long getPosition() = 0;

	virtual ~CursorImpl() {};
};

}

#endif /*CURSORIMPL_H_*/
