/**
 * @file iopcDb.h
 *
 * @author Petr Cermak
 *
 * @ingroup iopcdb
 * @defgroup iopcdb iopcdb
 */

#ifndef IOPCDB_H_
#define IOPCDB_H_

#include "iopccommon/trace.h"

namespace iopc {


class MetadataInitializer;

/**
 * @brief @c iopcdb library/configuration initialization class
 *
 * When using the @c iopcdb configuration, the library must be initialized
 * by calling the @a IopcDb::init() method and terminated by calling
 * the @a IopcDb::shutdown() method.
 *
 * @c iopccommon and iopcmeta libraries are initialized and terminated implicitly
 *
 * @ingroup iopcdb
 */
class IopcDb {

	public:
		/**
		 * @brief Initializes the @c iopcdb library/configuration
		 *
		 * @param traceMaskTTY @a LogWriter mask used for console output
		 * @param traceMask @a LogWriter mask used for log file output
		 * @param logFileName Name of the IOPC log file
		 * @param initDependencies If false is passed, the @c iopcmeta and @c iopcdb libraries
		 * are not initialized implicitly. They must be however initialized using their
		 * initialization routines before initializing @c iopcdb
		 */
		static void init(
				int traceMaskTTY = LogWriter::defaultTraceMaskTTY,
				int traceMask = LogWriter::defaultTraceMask,
				const std::string& logFileName = LogWriter::defaultFileName,
				bool initDependencies = true);
		/**
		 * @brief Initializes the @c iopcdb library/configuration. @a MetadataInitializer can be specified.
		 *
		 * @a metadataInitializer is called during the library initialization.
		 * After that the @a MetadataInitializer instance is deallocated.
		 *
		 * @param metadataInitializer The @a MetadataInitializer
		 * @param traceMaskTTY @a LogWriter mask used for console output
		 * @param traceMask @a LogWriter mask used for log file output
		 * @param logFileName Name of the IOPC log file
		 * @param initDependencies If false is passed, the @c iopcmeta and @c iopcdb libraries
		 * are not initialized implicitly. They must be however initialized using their
		 * initialization routines before initializing @c iopcdb
		 */
		static void init(
				MetadataInitializer* metadataInitializer,
				int traceMaskTTY = LogWriter::defaultTraceMaskTTY,
				int traceMask = LogWriter::defaultTraceMask,
				const std::string& logFileName = LogWriter::defaultFileName,
				bool initDependencies = true);

		/**
		 * @brief Terminates the @c iopcmeta library/configuration
		 *
		 * @param shutdownDependencies If false is passed, the @c iopcmeta and @c iopcdb libraries
		 * are not terminated implicitly. They must be however terminated using their
		 * termination routines after terminating @c iopcdb
		 */
		static void shutdown(bool shutdownDependencies = true);

};


}

#endif /* IOPCDB_H_ */
