/**
 * @file oracle10gConnectionImpl.h
 *
 * @author Josef Troch Josef Troch (<a href="mailto:josef.troch@email.cz">josef.troch@email.cz</a>) 2003-2004
 * @author Petr Cermak
 */

#ifndef ORACLE10GCONNECTION_H
#define ORACLE10GCONNECTION_H

//#include "oracle10gDatabase.h"
#include "iopcdb/connectionImpl.h" // parent class
#include "iopccommon/defsPlatform.h"
#include <string>
#include "oracle10gOci.h"

namespace iopc {

//class Oracle10gDatabaseImpl;

/*! \brief Abstraction of connection to an Oracle database using Oracle 10g OCI

	Methods of this class return true if OK, false otherwise (if not explicitly said something different).
	In case of connection error they throw exception ObjLibException_ConnectionError.

	\see Oracle10gSqlStatements, Oracle10gDatabase, Oracle10gCursor
 */
class DLL_EXTERNAL Oracle10gConnectionImpl : public ConnectionImpl
{

/*friend class Oracle10gCursor;*/
//friend int processOCIErrorGlobal(Oracle10gClassType classType, OCIEnv * ociEnvironment, OCIError * errorHandle, sword ociRetVal, const char * ociFunctionName, const char * myFunctionName, bool useEnvHandle);


public:
	 /*! Constructor. It should be called only from Oracle10gDatabase::Connect()

		 \note
		 %Connection to Oracle server is created before this function call in Oracle10gDatabase::Connect(). Resulting
		 \p serviceContext is then sent to this constructor.

		 \param db				Oracle10gDatabase this connection belongs to.
		 \param userName			User name used for connecting.
		 \param password			Password used for connecting.
		 \param serviceContext	Pointer to service context handle already created in Oracle10gDatabase::Connect().
	 */
	Oracle10gConnectionImpl(
		OCIEnv* ociEnv,
		const std::string& userName,
		const std::string& password,
		const std::string& dbName,
		bool autoCommit = false);


	//! Destructor - empty for now.
	virtual ~Oracle10gConnectionImpl();

	//! Commits actual transaction.
	virtual void commit();

	//! Performs rollback of actual transaction.
	virtual void rollback();

	/*! Creates savepoint with given name.
			\param name Name of savepoint to be created.
	 */
	virtual void savepoint(const std::string& name);

	/*! Performs rollback to savepoint with given name
			\param name Name of savepoint to rollback to.
	 */
	virtual void rollbackToSavepoint(const std::string& name);

	virtual void open();

	virtual void close();

	virtual void setAutoCommit(bool ac);

	virtual CursorImpl* getCursor(const std::string& sqlCommand);

	virtual void returnCursor(CursorImpl* cur);

	inline OCISvcCtx* getServiceContext() {
		return serviceContext;
	}

	inline OCIError* getErrorHandle() {
		return errorHandle;
	}

private:


	//! The service context pointer - represents one opened connection (created in Oracle10gDatabase::Connect()).
	OCISvcCtx* serviceContext;

	/*! Pointer to OCI Environment handle (see OCIEnvCreate() in Oracle documentation).
			Created in Oracle10gDatabase::Oracle10gDatabase() and copied here in constructor of this class.
	 */
	OCIEnv* ociEnvironment;

	/*! Pointer to OCI error handle to be passed to OCIErrorGet() for diagnostic information in the event of an error.
			Created in Oracle10gDatabase::Oracle10gDatabase() and copied here in constructor of this class.
	 */
	OCIError* errorHandle;

	bool autoCommit;

	std::string userName;
	std::string password;
	std::string dbName;

	void sqlNonQuery(const std::string& sqlCommand);

private:
		inline const std::string getOCIMsg(
		int ociRetVal,
		const std::string funcName,
		bool useEnvHandle = false) {

		return 	iopc::getOCIMsg(ociEnvironment, errorHandle, ociRetVal, funcName, useEnvHandle);
	}

};

} // namespace iopc

#endif //ORACLE10GCONNECTION_H
