/**
 * @file arcCacheItem.h
 *
 * @author Jan Hadrava
 * @date 2004
 * @ingroup iopclibcache
 */

#ifndef ARCCACHEITEM_H_
#define ARCCACHEITEM_H_


// project: POLiTe
// authors: Jan Hadrava 2004

/** \file */

#include "iopccommon/defs.h"
#include "iopcmeta/object.h"
#include "iopccommon/rwxLock.h"
#include "iopccommon/timeStamp.h"
#include "cacheLock.h"
#include "strategy.h"
#include "../persIdentification.h"

namespace iopc {

class ArcCacheBase;

/** Represents an object stored in cache using LRU strategy. Usually the item is part of a double linked list.
 *
 * @ingroup iopclibcache
 * */
class DLL_EXTERNAL ArcCacheItem: public CacheLock {
public:
	/** Enumeration of item states. */
	enum State {
		ST_BORN,	/**< An object without persistent copy. */
		ST_ADULT,	/**< An object with persistent copy. */
		ST_DOOMED,	/**< DatabaseObject scheduled for deletion. */
		ST_ZOMBIE,	/**< Remnant of an item. */
		ST_BUSY,	/**< Item being processed. */
		ST_HIST		/**< History record. */
	};
public:
	ArcCacheItem();
	ArcCacheItem(ArcCacheBase &cache, const Strategy& strat, const PersIdentification persId, DatabaseObject* obj = NULL, State state = ArcCacheItem::ST_BORN);
	~ArcCacheItem();
private:	/* hiding */
	ArcCacheItem(const ArcCacheItem& );
	ArcCacheItem &operator=(const ArcCacheItem& );
public:
	DatabaseObject& getObject();
	CacheLock& addLock(bool rdonly);
	void rmLock();
	/** Replaces the object represented by the item. */
	void replaceObject(DatabaseObject* obj);
	/** Removes item from list. */
	void remove();
	/** Appends an item. */
	void addNext(ArcCacheItem &item);
	/** Prepends an item. */
	void addPrev(ArcCacheItem &item);
	/** Releases the object being represented by the item. */
	void release();
	/** Set frequency/recency. */
	void setFrequent(bool frequent = true);
	/** Inspect frequency/recency. */
	bool isFrequent();
public:
	RWXLock lock;
	ArcCacheBase* cache;	/**< Owner of the item. */
	Strategy strat;
	PersIdentification persId;	/**< ID of the object. */
	DatabaseObject* objectPtr;		/**< The object represented by the item. */
	State state;			/**< State of the item. */
	ArcCacheItem* next;		/**< Next item. */
	ArcCacheItem* prev;		/**< Previous item. */
	bool isLocked;
	TimeStamp timeStamp;		/**< Time of the last access to the item. */
	bool frequent;		/**< indicates, whether item was accessed more than once (whether is frequent xor recent). */
};

}/*namespace iopc*/


#endif /*ARCCACHEITEM_H_*/
