/**
 * @file strategy.h
 *
 * @author Jan Hadrava
 * @date 2004
 * @ingroup iopclibcache
 */


#ifndef STRATEGY_H_
#define STRATEGY_H_

// project: POLiTe
// authors: Jan Hadrava 2004


#include "../databaseObject.h"
#include "types.h"
#include "iopccommon/defs.h"

namespace iopc {

typedef void (*WakeUpMntrFunction)(void* data, long costChange);
typedef long (*CostFunction)(DatabaseObject& obj);

/**
 * @brief A strategy class
 *
 * @ingroup iopclibcache
 */
class DLL_EXTERNAL Strategy {
public:
	static class Strategy eager;
	static class Strategy lazy;
	static class Strategy async;
	static class Strategy xAccess;
	static class Strategy &defaultStrategy;
public:
	Strategy(bool loadWait = true, bool loadLocking = false,
		enum UpdatingStrategy uStrat = USDeferred, enum ReadingStrategy rStrat = RSCache,
		void* wakeUpData = NULL, WakeUpMntrFunction wakeUpFunction = NULL, CostFunction costFunction = &constCost);
	virtual ~Strategy();
public:
	inline virtual bool getLoadWait() const;
	inline virtual bool getLoadLocking() const;
	inline virtual UpdatingStrategy getUpdatingStrategy() const;
	inline virtual ReadingStrategy getReadingStrategy() const;
	inline virtual void wakeUpMaintainer(long cost) const;
	inline virtual bool hasMaintainer() const;
	inline virtual int cost(DatabaseObject &obj) const;
	inline virtual void set(bool loadWait, bool loadLocking, UpdatingStrategy uStrat, ReadingStrategy rStrat);
	inline virtual void offline();
	static long memoryCost(DatabaseObject &obj);
	static long constCost(DatabaseObject &obj);
private:
	bool loadWait;
	bool loadLocking;
	UpdatingStrategy uStrat;
	ReadingStrategy rStrat;

	void* wakeUpData;
	WakeUpMntrFunction wakeUpFunction;
	CostFunction costFunction;
};

inline bool Strategy::getLoadWait() const
{
	return loadWait;
}

inline bool Strategy::getLoadLocking() const
{
	return loadLocking;
}

inline UpdatingStrategy Strategy::getUpdatingStrategy() const
{
	return uStrat;
}

inline ReadingStrategy Strategy::getReadingStrategy() const
{
	return rStrat;
}

inline void Strategy::wakeUpMaintainer(long costChange) const
{
	if (wakeUpFunction != NULL) {
		(*wakeUpFunction)(wakeUpData, costChange);
	}
}

inline bool Strategy::hasMaintainer() const
{
	return wakeUpFunction != NULL;
}
inline int Strategy::cost(DatabaseObject &obj) const
{
	if (costFunction == NULL) {
		return 1;
	}
	return costFunction(obj);
}

inline void Strategy::set(bool loadWait, bool loadLocking, UpdatingStrategy uStrat, ReadingStrategy rStrat)
{
	this->loadWait = loadWait;
	this->loadLocking = loadLocking;
	this->uStrat = uStrat;
	this->rStrat = rStrat;
}

inline void Strategy::offline()
{
	costFunction = NULL;
	wakeUpFunction = NULL;
}

}/*namespace polite*/




#endif /*STRATEGY_H_*/
