/**
 * @file keyValues.h
 *
 * @author Petr Cermak
 *
 * @ingroup iopclib
 */

#ifndef KEYVALUES_H_
#define KEYVALUES_H_

#include "iopccommon/defsPlatform.h"
#include "iopcmeta/enhancedTypeBase.h"
#include "iopcmeta/enhancedNumber.h"
#include "iopcmeta/enhancedString.h"
#include <string>
#include <map>

namespace iopc {

/**
 * @brief As OID represents the identity of @a OidObject-s, @a KeyValues represent the identity of all other persistent
 * non-OID objects derived from @a DatabaseObject.
 *
 * @a KeyValues contain a list of key-value pairs which represents any single-attribute or composite (multi-attribute) key.
 *
 * @a KeyValues can be used as a key in STL containers.
 *
 * @ingroup iopclib
 */
class DLL_EXTERNAL KeyValues
{
private:
	std::map<std::string, EnhancedTypeBase*> values;

	void copyFrom(const KeyValues& src);
public:
	KeyValues();
	KeyValues(const KeyValues& src);

	/**
	 * @brief Key values iterator
	 */
	typedef std::map<std::string, EnhancedTypeBase*>::const_iterator Iterator;

	/**
	 * @brief The iterator begin() method
	 */
	inline Iterator begin() const {
		return values.begin();
	}

	/**
	 * @brief The iterator end() method
	 */
	inline Iterator end() const {
		return values.end();
	}

	/**
	 * @brief Returns number of the key values stored in the current object.
	 */
	inline int size() const {
		return values.size();
	}

	/**
	 * @brief Adds a new ENumber<T> value
	 *
	 * @param key Name of the key (i.e. attribute name)
	 * @param value The value
	 */
	template<typename T>
	void add(std::string key, ENumber<T> value) {
		add(key, (EnhancedTypeBase&) value);
	}

	/**
	 * @brief Adds a new EString<T> value
	 *
	 * @param key Name of the key (i.e. attribute name)
	 * @param value The value
	 */
	template<typename T>
	void add(std::string key, EStr<T> value) {
		add(key, (EnhancedTypeBase&) value);
	}

	/**
	 * @brief Adds a new C++ built-in type or STL string type value
	 *
	 * @param key Name of the key (i.e. attribute name)
	 * @param value The value
	 */
	template<typename T>
	void add(std::string key, T value);

	/**
	 * @brief Adds a new Enhanced data type value
	 *
	 * @param key Name of the key (i.e. attribute name)
	 * @param value The value, a copy is created
	 */
	void add(std::string key, EnhancedTypeBase& value);

	/**
	 * @brief Adds a new Enhanced data type value
	 *
	 * @param key Name of the key (i.e. attribute name)
	 * @param value The value, a copy is created
	 */
	void add(std::string key, EnhancedTypeBase* value);

	/**
	 * @brief Adds a new Enhanced data type value
	 *
	 * @param key Name of the key (i.e. attribute name)
	 * @param value A pointer to the value, no copy is created.
	 */
	void addNoCopy(std::string key, EnhancedTypeBase* value);

	/**
	 * @brief Returns a value stored under the specified key
	 *
	 * @throws IopcNotFoundException if no such key value could be found
	 */
	EnhancedTypeBase& get(const std::string& key) const;

	/**
	 * @brief Determines whether the @a KeyValues contain any value of specified key
	 */
	bool contains(const std::string& key) const;

	/**
	 * @brief Clears the container
	 *
	 * Deallocates all values - copied and even those added using @a addNoCopy
	 */
	void clear();

	virtual ~KeyValues();

	KeyValues& operator=(const KeyValues& src);
	bool operator<(const KeyValues& id) const;	/* needed by STL containers */
	bool operator==(const KeyValues& id) const;
	bool operator!=(const KeyValues& id) const;

	int compare(const KeyValues& id) const;

	std::string toString() const;
};



}


#endif /*KEYVALUES_H_*/

