/**
 * @file dbType.h
 *
 * @author Petr Cermak
 *
 * @ingroup iopclib
 */

#ifndef DBTYPE_H_
#define DBTYPE_H_

#include "iopccommon/defsPlatform.h"
#include "../oidObject.h"
#include "iopcdb/connection.h"
#include "iopcmeta/typeDesc.h"
#include "iopccommon/rwLock.h"
#include "../keyValues.h"
#include "mappingStrategy.h"
//#include "meta/complexType.h"
#include <string>
#include <vector>

namespace iopc {

class Driver;
class ObjectMappingStrategy;
class ORMappingStrategy;


/**
 * @brief Provides O/R mapping services for a @a Type
 *
 * This class 'wraps' persistent @a Type-s (descendants of @a DatabaeObject) and provides
 * O/R mapping services for them.
 *
 * It implements the table-level part of the mapping algorithm described in the Thesis.
 * Attribute-level part is implemented in @a ObjectMappingStrategy and @a ORMappingStrategy classes.
 *
 * When one of the CRUD methods of DbType is invoked, the program iterates
 * through the inheritance hierarchy of the related class type according to the algorithm and calls
 * the ObjectMappingStrategy or ORMappingStrategy to perform the attribute-level
 * operations. The strategies in turn call the database drivers' ObjectStatementsFeature
 * or ORStatementsFeature to generate SQL statements that will be executed.
 *
 * @see ObjectMappingStrategy
 * @see ORMappingStrategy
 * @see DbTypeManager
 *
 * @ingroup iopclib
 */
class DLL_EXTERNAL DbType {
	friend class PersIdentification;
public:
	DbType();

	/**
	 * @brief Initializes the @a DbType instnace
	 *
	 * Checks associated type metadata and object model and pre-generated
	 * required lists that are used during the mapping algorithm execution.
	 */
	void init(const Type& type);

	/**
	 * @brief Second phase of the initialization routine
	 *
	 * Executed after all @a DbType-s have finished the first phase (@a init())
	 */
	void postInit();

public:

	typedef std::vector<const Attribute*> PersistentAttributesList;
	typedef PersistentAttributesList::const_iterator PersistentAttributesIterator;

	typedef std::set<const Attribute*> KeyAttributesSet;
	typedef KeyAttributesSet::const_iterator KeyAttributesIterator;

	typedef std::set<DbType*> AllParentsSet;
	typedef std::vector<DbType*> AllParentsList;
	typedef AllParentsList::const_iterator AllParentsIterator;
	typedef AllParentsList::const_reverse_iterator AllParentsReverseIterator;

	inline bool operator!=(const DbType& src) const { return *type != *src.type; }
	inline bool operator==(const DbType& src) const { return *type == *src.type; }
	inline bool operator<(const DbType& src) const { return *type < *src.type; }

	/**
	 * @brief Loads a database object by sending multiple select statements to load data from tables its attributes are mapped into
	 *
	 * Doesn't use simple views. Usable even for @a OidObject-s if we don't want to use the associated simple view.
	 *
	 * @param conn Connection from which to load the database object
	 * @param keyVals Key values describing identity of the object to be loaded
	 * @param wait Specifies whether the operation should wait for database locks to be unlocked.
	 * @param lock Specifies whether the persistent copy should be locked in DB exlusively when loaded.
	 *
	 * @returns A new instance of @a DatabaseObject descendant filled with loaded data.
	 */
	DatabaseObject* loadDatabaseObject(Connection& conn, const KeyValues& keyVals, bool wait, bool lock);

	/**
	 * @brief Loads a database object from an open cursor
	 *
	 * The data to be fetched are expected to correspond to the @a getAllPersistentAttributes() list.
	 * Fetches exactly one row.
	 *
	 * @param cur Cursof from which to load the data
	 *
	 * @returns A new instance of @a DatabaseObject descendant filled with data from the cursor.
	 */
	DatabaseObject* loadDatabaseObject(Cursor& cur);

	/**
	 * @brief Loads an @a OidObject by selecting from its associated simple view
	 *
	 * If @a lock = true is specified, the call is delegated to @a loadDatabaseObject(conn, keyVals, wait, lock) as
	 * the data need to be locked in all underlying tables. (-> sends multiple SELECT FOR UPDATE or similar statements)
	 *
	 * @param conn Connection from which to load the database object
	 * @param oid OID of the object to be loaded
	 * @param wait Specifies whether the operation should wait for database locks to be unlocked.
	 * @param lock Specifies whether the persistent copy should be locked in DB exlusively when loaded.
	 *
	 * @returns A new instance of @a DatabaseObject descendant filled with loaded data.
	 */
	OidObject* loadOidObject(Connection& conn, const OidType& oid, bool wait, bool lock);

	/**
	 * @brief Deletes an @a OidObject of specified OID from the underlying database
	 *
	 * @param conn Connection used to delete the object
	 * @param oid OID of the object to delete
	 */
	void deleteOidObject(Connection& conn, const OidType& oid);

	/**
	 * @brief Deletes an @a OidObject of specified identity from the underlying database
	 *
	 * @param conn Connection used to delete the object
	 * @param keyVals Key values that identify of the object to delete
	 */
	void deleteDatabaseObject(Connection& conn, const KeyValues& keyVals);


	/**
	 * @brief Updates persistent copy
	 *
	 * Updates @a DatabaseObject-s as well as @a OidObject-s
	 *
	 * @param conn Connection used for the update
	 * @param obj Local copy that is used to update the persistent copy in the database.
	 */
	void updateObject(Connection& conn, DatabaseObject& obj);

	/**
	 * @brief Inserts a persistent object into a database
	 *
	 * Updates @a DatabaseObject-s as well as @a OidObject-s
	 *
	 * @param conn Connection used for the DB operation.
	 * @param obj Persistent objects that is inserted into the database
	 */
	void insertObject(Connection& conn, DatabaseObject& obj);

	/**
	 * @brief Creates a @a KeyValues instance from attribute values in the object @a obj
	 *
	 * @param obj Object from which to crate the @a Keyvalues instance
	 */
	KeyValues getKeyValuesForObject(const DatabaseObject& obj) const;

	/**
	 * @brief Returns the @a Type associated with this @a DbType
	 */
	inline const Type& getType() const { return *type; }

	virtual ~DbType();

	/**
	 * @brief Mapping type used for the current type
	 *
	 * [db.mapping.type] metadata
	 */
	enum Mapping {
		MAPPING_HORIZONTAL =0,
		MAPPING_VERTICAL,
		MAPPING_FILTERED,
		/** @brief ADT mapping */
		MAPPING_OBJECT
	};

	inline Mapping getMappingType() const { return mapping; }

	/**
	 * @brief Returns a list of persistent parent classes (even indirect) that have a corresponding database table.
	 *
	 * This includes only classes that use horizontal or vertical mapping. Classes that use filtered
	 * mapping do not have any associated database table as their columns are inserted into one
	 * of their ancestors. Because ADT-mapped classes are handled in a different way, they
	 * are not inserted into this list either. The list represents tables that will be accessed when
	 * performing a CRUD operation on an instance of the current class. Only one database
	 * operation is needed to modify database data belonging to an instance of any class in an
	 * ADT-mapped class subgraph.
	 *
	 * The list is created by level-order walking through the parent hierarchy starting with
	 * parents of the current class first. If the process encounters a horizontally-mapped class,
	 * its parents are not processed (because horizontally-mapped classes store all, even
	 * inherited attributes into one table). If the current class derives from OidObject, and if
	 * OidObject does not use ADT mapping, OidObject is also appended to this list.
	 */
	inline const AllParentsList& getAllParents() const { return allParents; }

	/**
	 * @brief Returns a set of persistent parent classes (even indirect) that have a corresponding database table.
	 *
	 * Same functionality as @a getAllParents(), but a set instead of vector is returned
	 *
	 * @see getAllParents()
	 */
	inline const AllParentsSet& getAllParentsSet() const { return allParentsSet; }

	/**
	 * @brief Returns union of the @a getPersistentAttributes() and @a getInheritedAttributes() lists.
	 */
	inline const PersistentAttributesList& getAllPersistentAttributes() const { return allPersistentAttributes; }

	/**
	 * @brief list of key attributes that are inherited or declared in the current class.
	 *
	 * If the current class derives from OidObject, the OID attribute is included in
	 * the list. A class cannot re-define key attributes if they were already defined in one of its
	 * ancestors.
	 */
	inline const KeyAttributesSet& getKeyAttributes() const { return keyAttributes; }

	/**
	 * @brief Returns a list of persistent attributes declared in the current class.
	 *
	 * Developers may declare some of the class attributes as transient. Transient attributes are
	 * ignored by the persistence layer.
	 */
	inline const PersistentAttributesList& getPersistentAttributes() const { return persistentAttributes; }

	/**
	 * @brief List of persistent attributes (retrieved from PersistentAttributes) inherited from all persistent ancestors.
	 */
	inline const PersistentAttributesList& getInheritedAttributes() const { return inheritedAttributes; }

	/**
	 * @brief Returns true if the type is descendant of an @a OidObject (or is @a OidObject), false otherwise
	 */
	inline bool isOidManaged() const { return oidManaged; }

	/**
	 * @brief Returns the current mapping strategy
	 *
	 * @returns @a ObjectMappingStrategy, if ADT (object) mapping is used, @a ORMappingStrategy if horizontal, vertical or filtered
	 * mapping is used.
	 */
	inline MappingStrategy& getMappingStrategy() const { return *strategy; }

	/**
	 * @brief Returns @a ObjectMappingStrategy if ADT (object) mapping is used
	 *
	 * @throws IopcIncorrectUsageException if other mapping type is used
	 */
	ObjectMappingStrategy& getObjectMappingStrategy() const;

	/**
	 * @brief Returns @a ORMappingStrategy if horizontal, vertical or filtered mapping is used
	 *
	 * @throws IopcIncorrectUsageException if other mapping type is used
	 */
	ORMappingStrategy& getORMappingStrategy() const;
private:

	void insertOne(Connection& conn, DatabaseObject& obj);
	void updateOne(Connection& conn, DatabaseObject& obj);
	void deleteOne(Connection& conn, const OidType& oid);
	void deleteOne(Connection& conn, const KeyValues& keyVals);
	void loadOne(Connection& conn, DatabaseObject* obj, const KeyValues& keyVals, bool wait, bool lock);


	Mapping mapping;

//	DbType& operator=(DbType& src);
private:
	const Type* type;
	bool initialized;

	/** All (even indirect) parents of this type that have an associated table in the underlying database */
	AllParentsList allParents;
	AllParentsSet allParentsSet;

	/** List of persistent attributes */
	PersistentAttributesList persistentAttributes;

	/** List of inherited attributes */
	PersistentAttributesList inheritedAttributes;

	/** List of all persistent attributes (even inherited) */
	PersistentAttributesList allPersistentAttributes;

	/** List of keys **/
	KeyAttributesSet keyAttributes;


	//RWLock& lock;

	bool oidManaged;

	MappingStrategy* strategy;
};

}


#endif /*DBTYPE_H_*/
