/**
 * @file mappingStrategy.h
 *
 * @author Petr Cermak
 *
 * @ingroup iopclib
 */

#ifndef MAPPINGSTRATEGY_H_
#define MAPPINGSTRATEGY_H_

#include "../oidObject.h"
namespace iopc {

class Connection;
class Cursor;
class DatabaseObject;
class KeyValues;
class Type;
class DbType;

/**
 * @brief Declares methods that the @a ObjectMappingStrategy and @a ORMappingStrategy implement
 *
 * These methods implement the attribute-level part of the O/R mapping algorithm.
 * Each @a DbType contains an instance of one of these strategies depending on what mapping type it uses.
 * The strategy instances are aware of the @a Type and @a DbType they are associated to.
 *
 * @ingroup iopclib
 */
class MappingStrategy
{
public:
	/**
	 * @brief Validates the associated persistent object
	 *
	 * Checks all mapping-related metadata and pre-generates lists of mapped attributes or other
	 * data it needs during the mapping algorithm execution.
	 */
	virtual void init() =0;
	/**
	 * @brief Second phase of the initialization routine.
	 *
	 * Executed after all strategies and @a DbType-s have finished the first phase (@a DbPtr::init(), MappingStrategy::init())
	 */
	virtual void postInit() =0;

	/**
	 * @brief Loads attributes declared in the associated type from its associated mapping table or type
	 *
	 * @param conn Connection from which to load the attributes
	 * @param obj Object whose attributes will be loaded from conn
	 * @param keyVals Identifies the object whose attributes to load
	 * @param wait Specifies whether the operation should wait for database locks to be unlocked.
	 * @param lock Specifies whether the persistent copy should be locked in DB exlusively when loaded.
	 */
	virtual void loadOne(Connection& conn, DatabaseObject* obj, const KeyValues& keyVals, bool wait, bool lock) =0;

	/**
	 * @brief Loads all attributes from the associated types' simple view
	 *
	 * @param conn Connection from which to load the attributes
	 * @param oid OID of the object to be laoded
	 * @param wait Specifies whether the operation should wait for database locks to be unlocked.
	 * @param lock Specifies whether the persistent copy should be locked in DB exlusively when loaded.
	 * @returns Allocates, loads and returns a new @a OidObject instance
	 */
	virtual OidObject* viewLoadAll(Connection& conn, const OidType& oid, bool wait, bool lock) =0;

	/**
	 * @brief Updates columns from attributes declared in the associated type in its associated mapping table or ADT type instance
	 *
	 * @param conn Connection used to update the values
	 * @param obj Obj containing attributes that update the database values
	 */
	virtual void updateOne(Connection& conn, DatabaseObject& obj) =0;

	/**
	 * @brief Deletes an OidObject identified by OID from the associated table
	 * ord deletes the associated ADT type instance.
	 *
	 * @param conn Connection used to delete the object
	 * @param oid OID of the object to delete
	 */
	virtual void deleteOne(Connection& conn, const OidType& oid) =0;

	/**
	 * @brief Deletes an OidObject identified by @ KeyValues from the associated table
	 * or deletes the associated ADT type instance.
	 *
	 * @param conn Connection used to delete the object
	 * @param keyVals @a KeyValues of the object to delete
	 */
	virtual void deleteOne(Connection& conn, const KeyValues& keyVals) =0;

	/**
	 * @brief Inserts values of attributes declared in the associated type to associated table
	 * or creates a new associated ADT type instance.
	 *
	 * @param conn Connection used to perform the operation
	 * @param obj Object containing the values to insert.
	 */
	virtual void insertOne(Connection& conn, DatabaseObject& obj) =0;

	/**
	 * @brief Returns the associated @a DbType instance
	 */
	virtual DbType& getDbType() const =0;

	/**
	 * @brief Returns the associated @a Type instance
	 */
	virtual const Type& getType() const =0;

	virtual ~MappingStrategy() {};
protected:

	void mapKeyAttributes(Cursor* cur, int startIndex, DatabaseObject& obj);
	void mapKeyAttributes(Cursor* cur, int startIndex, const KeyValues& keyVals);
	void mapKeyAttributes(Cursor* cur, int startIndex, const OidType& oid);
};

}

#endif /*MAPPINGSTRATEGY_H_*/
