/**
 * @file ORMappingStrategy.h
 *
 * @author Petr Cermak
 *
 * @ingroup iopclib
 */

#ifndef ORMAPPINGSTRATEGY_H_
#define ORMAPPINGSTRATEGY_H_

#include "mappingStrategy.h"
#include "iopcmeta/type.h"
#include "dbType.h"

namespace iopc {

class Attribute;

/**
 * @brief Implements attribute-level part of the O/R mapping algorithm for the horizontal, vertical and filtered mapping types
 *
 * @ingroup iopclib
 */
class ORMappingStrategy : public MappingStrategy
{
private:
	DbType* dbType;
public:
	ORMappingStrategy(DbType* dbType) : dbType(dbType) {};
	virtual void init();
	virtual void postInit();
	virtual void loadOne(Connection& conn, DatabaseObject* obj, const KeyValues& keyVals, bool wait, bool lock);
	virtual OidObject* viewLoadAll(Connection& conn, const OidType& oid, bool wait, bool lock);
	virtual void updateOne(Connection& conn, DatabaseObject& obj);
	virtual void deleteOne(Connection& conn, const OidType& oid);
	virtual void deleteOne(Connection& conn, const KeyValues& keyVals);
	virtual void insertOne(Connection& conn, DatabaseObject& obj);
	virtual inline DbType& getDbType() const {return *dbType; }
	virtual inline const Type& getType() const { return dbType->getType(); }
public:

	/**
	 * @brief Classifies the attributes in the @a getMappedAttributes() list
	 */
	enum ColSource {
		/** @brief Persistent attributes of the current class. (PersistentAttributes) */
		COLSOURCE_ATTR =0,
		/** @brief Inherited KeyAttributes - even if the class does not employ horizontal mapping */
		COLSOURCE_INHERITEDPK,
		/** @brief If the current class uses horizontal mapping, the list includes non-key inherited
		 * persistent attributes from the InheritedAttributes list. Attributes inherited
		 * from the OidObject are excluded as the Oid - classname catalogue (a table
		 * associated with OidObject) is accessed even when using horizontal mapping.*/
		COLSOURCE_INHERITED,
		/** @brief Persistent attributes from classes that use filtered mapping - persistent attributes
	     * from classes in the FilteredTypes list. */
		COLSOURCE_FILTERED
	};

	typedef std::vector<std::pair<const Attribute*, ColSource> > MappedAttributesList;
	typedef MappedAttributesList::const_iterator MappedAttributesIterator;

	typedef std::set<const DbType*> FilteredTypesList;
	typedef FilteredTypesList::const_iterator FilteredTypesIterator;
public:

	/**
	 * @brief Returns a list of attributes that are physically mapped into table associated with @a getType()
	 *
	 * In fact they represent columns of this table. The list may include attributes from other classes.
	 * Attributes are categorized by their origin:
	 * <ul>
	 * <li>Persistent attributes of the current class. (PersistentAttributes)</li>
	 * <li>If the current class uses horizontal mapping, the list includes non-key inherited
	 * persistent attributes from the InheritedAttributes list. Attributes inherited
	 * from the OidObject are excluded as the Oid - classname catalogue (a table
	 * associated with OidObject) is accessed even when using horizontal mapping.</li>
	 * <li>Persistent attributes from classes that use filtered mapping - persistent attributes
	 * from classes in the FilteredTypes list.</li>
	 * <li>Inherited KeyAttributes - even if the class does not employ horizontal mapping</li>
	 * </ul>
	 *
	 */
	inline const MappedAttributesList& getMappedAttributes() const { return mappedAttributes; }

	/**
	 * @brief Returns true if the type has a database table associated (false for filtered mapping, true for other mapping types)
	 */
	inline bool hasDbTable() const { return hasTable; }

	/**
	 * @brief Returns destination type for filtered-mapping classes.
	 *
	 * For other mapping types, behaviour of this method is undefined.
	 */
	inline DbType& getInsertToType() const { return *insertToType; }

private:

	MappedAttributesList mappedAttributes;

	FilteredTypesList filteredTypes;

	void addFilteredType(DbType& type);

	bool hasFilteredColumns;
	bool hasTable;
	DbType* insertToType;
};

}

#endif /*ORMAPPINGSTRATEGY_H_*/
