/**
 * @file iopcMeta.h
 *
 * @author Petr Cermak
 *
 * @ingroup iopcmeta
 * @defgroup iopcmeta iopcmeta
 */

#ifndef IOPCMETA_H_
#define IOPCMETA_H_

#include "metadataInitializer.h"
#include "iopccommon/trace.h"

namespace iopc {

/**
 * @brief @c iopcmeta library/configuration initialization class
 *
 * When using the @c iopcmeta configuration, the library must be initialized
 * by calling the @a IopcMeta::init() method and terminated by calling
 * the @a IopcMeta::shutdown() method.
 *
 * @c iopccommon library is initialized and terminated implicitly
 *
 * @ingroup iopcmeta
 */
class IopcMeta {
	private:
		static MetadataInitializer* initializer;


	public:
		/**
		 * @brief Initializes the @c iopcmeta library/configuration
		 *
		 * @param traceMaskTTY @a LogWriter mask used for console output
		 * @param traceMask @a LogWriter mask used for log file output
		 * @param logFileName Name of the IOPC log file
		 */
		static void init(
				int traceMaskTTY = LogWriter::defaultTraceMaskTTY,
				int traceMask = LogWriter::defaultTraceMask,
				const std::string& logFileName = LogWriter::defaultFileName);

		/**
		 * @brief Initializes the @c iopcmeta library/configuration. @a MetadataInitializer can be specified.
		 *
		 * @a metadataInitializer is called during the library initialization.
		 * After that the @a MetadataInitializer instance is deallocated.
		 *
		 * @param metadataInitializer The @a MetadataInitializer
		 * @param traceMaskTTY @a LogWriter mask used for console output
		 * @param traceMask @a LogWriter mask used for log file output
		 * @param logFileName Name of the IOPC log file
		 */
		static void init(
				MetadataInitializer* metadataInitializer,
				int traceMaskTTY = LogWriter::defaultTraceMaskTTY,
				int traceMask = LogWriter::defaultTraceMask,
				const std::string& logFileName = LogWriter::defaultFileName);

		/**
		 * @brief Terminates the @c iopcmeta library/configuration
		 */
		static void shutdown();

};


}

#endif /*IOPCMETA_H_*/
