/**
 * @file persIdentification.cpp
 *
 * @author Jan Hadrava
 * @author Petr Cermak
 *
 * @ingroup iopclib
 */

#include "persIdentification.h"
#include "iopccommon/exceptionBase.h"
#include "iopccommon/trace.h"
#include "types/dbType.h"
#include "types/dbTypeManager.h"
#include <sstream>
#include <ostream>

using namespace std;

namespace iopc {

//const PersIdentification PersIdentification::nil;

PersIdentification::PersIdentification()
:  /*type(&type),*/ valid(false), usable(false), dbType(DbTypeManager::getDbType(TypeDesc<OidObject>::getType())), loadRealType(false), realDbType(NULL)
{
	valid = false;

}

PersIdentification::PersIdentification(const Type& type)
:  /*type(&type),*/ valid(false), usable(true), dbType(DbTypeManager::getDbType(type)), loadRealType(false), realDbType(NULL)
{
	if (type.isTypeOf<OidObject>())
	{
		idType = OID;
	} else {
		idType = KEYVALUES;
	}
}

PersIdentification::PersIdentification(const PersIdentification& src)
: keyValues(src.keyValues), /*type(src.type),*/ valid(src.valid), usable(src.usable), idType(src.idType), oid(src.oid), dbType(src.dbType), loadRealType(src.loadRealType), realDbType(src.realDbType)
{
}

PersIdentification::PersIdentification(const Type& type, const OidType& oid, bool loadRealType)
: /*type(&type),*/ valid(true), usable(true), dbType(DbTypeManager::getDbType(type)), loadRealType(loadRealType), realDbType(NULL)
{
	this->oid = oid;
	idType = OID;
}

PersIdentification::PersIdentification(const Type& type, const KeyValues& keyValues)
: keyValues(keyValues), /*type(&type),*/ valid(true), usable(true), dbType(DbTypeManager::getDbType(type)), loadRealType(false), realDbType(NULL)
{
	IOPC_FUNC_INVOKED;
	if (keyValues.size() == 1 &&
			keyValues.contains("oid") &&
			keyValues.get("oid").getType().is<OidType>()
	) {
		this->oid = (OidType&) keyValues.get("oid");
		loadRealType = true;
		idType = OID;
		IOPC_FUNC_ENDED;
		return;
	}
	idType = KEYVALUES;
	IOPC_FUNC_ENDED;
}

PersIdentification::~PersIdentification()
{

}

PersIdentification& PersIdentification::operator=(const PersIdentification& src)
{
	if (this != &src) {
		//type = src.type;
		dbType = src.dbType;
		idType = src.idType;
		oid = src.oid;
		keyValues = src.keyValues;
		valid = src.valid;
		loadRealType = src.loadRealType;
		realDbType = src.realDbType;
		usable = src.usable;
	}
	return *this;
}

bool PersIdentification::operator<(const PersIdentification& id) const
{
	return compare(id) < 0;
}

int PersIdentification::compare(const PersIdentification& id) const
{
	/*if (idType == KEYVALUES) {
		if (dbType < id.dbType) return -1;

		if (dbType != id.dbType) return 1;
	}*/

	// dbType == id.dbType    vvv

	// this.valid < id.valid <= (this.valid && !id.valid)
	if (!isValid()) {
		if (!id.isValid()) {
			return 0;
		} else {
			return 1;
		}
	} else if (!id.isValid()) {
		return -1;
	}

	// valid && id.valid    vvv

	if (idType == KEYVALUES) {
		return keyValues.compare(id.keyValues);
	} else {
		return oid.compare(id.oid);
	}
}

bool PersIdentification::operator==(const PersIdentification& id) const
{
	return compare(id) == 0;
}

bool PersIdentification::operator!=(const PersIdentification& id) const
{
	return compare(id) != 0;
}

// No need to do locking.. It can't get into inconsistent state
DbType& PersIdentification::getDesiredDbType(Connection& conn) const
{
	if (loadRealType) {
		if (realDbType != NULL) {
			return *realDbType;
		}
		realDbType = &DbTypeManager::getDbType(DbTypeManager::getType(conn, oid));
		return *realDbType;
	} else {
		return dbType;
	}
}

DatabaseObject* PersIdentification::load(Cursor& cur) const
{
	IOPC_FUNC_INVOKED;
	DatabaseObject* objectPtr = dbType.loadDatabaseObject(cur);

	if (objectPtr) {
		objectPtr->resetDirty();
	}
	IOPC_FUNC_ENDED;
	return objectPtr;
}

DatabaseObject* PersIdentification::load(Connection& conn, bool wait, bool lock) const
{
	if (!isValid())
		IOPC_LOG_THROW(IopcIncorrectUsageException());
	if (idType == OID) {
		IOPC_FUNC_INVOKED;
		OidObject* objectPtr = NULL;
		DbType& dbType = getDesiredDbType(conn);
		objectPtr = dbType.loadOidObject(conn, oid, wait, lock);
		objectPtr->oid = oid;
		objectPtr->className = dbType.getType().getQualifiedName();

		// since we modified some attributes, we need to reset the dirty bit.
		objectPtr->resetDirty();
		IOPC_FUNC_ENDED;
		return objectPtr;
	} else {
		IOPC_FUNC_INVOKED;
		DatabaseObject* objectPtr = dbType.loadDatabaseObject(conn, keyValues, wait, lock);
		// since we modified some attributes, we need to reset the dirty bit.
		objectPtr->resetDirty();
		IOPC_FUNC_ENDED;
		return objectPtr;
	}
	//objectPtr->setPersistent(conn);
	//return objectPtr;
}

void PersIdentification::remove(Connection& conn) const
{
	IOPC_FUNC_INVOKED;
	if (!isValid())
		IOPC_LOG_THROW(IopcIncorrectUsageException());

	if (idType == OID) {
		getDesiredDbType(conn).deleteOidObject(conn, oid);
	} else {
		dbType.deleteDatabaseObject(conn, keyValues);
	}
}

void PersIdentification::update(Connection& conn, DatabaseObject& obj) const
{
	IOPC_FUNC_INVOKED;
	if (!isValid())
		IOPC_LOG_THROW(IopcIncorrectUsageException());

	if (obj.isOidObject()) {
		OidObject& oidObj = dynamic_cast<OidObject&>(obj);
		oidObj.serialId = DbTypeManager::getNextSerialId(conn);
		getDesiredDbType(conn).updateObject(conn, oidObj);
	} else {
		dbType.updateObject(conn, obj);
	}
}

void PersIdentification::insert(Connection& conn, DatabaseObject& obj) const
{
	IOPC_FUNC_INVOKED;
	if (!isValid())
			IOPC_LOG_THROW(IopcIncorrectUsageException());
	if (obj.isOidObject()) {
		getDesiredDbType(conn).insertObject(conn, obj);
	} else {
		dbType.insertObject(conn, obj);
	}
}

void PersIdentification::initIdentity(Connection& conn, DatabaseObject& obj)
{
	IOPC_FUNC_INVOKED;
	if (obj.isOidObject()) {
		OidObject& oidObj = dynamic_cast<OidObject&>(obj);
		oid = DbTypeManager::getNextOid(conn);
		oidObj.oid = oid;
		oidObj.serialId = DbTypeManager::getNextSerialId(conn);
		oidObj.className = getType().getQualifiedName();
	} else {
		keyValues = dbType.getKeyValuesForObject(obj);
	}

	valid = true;
	IOPC_FUNC_ENDED;
}

bool PersIdentification::versionMatches(Connection& conn, OidObject& obj) const
{
	IOPC_FUNC_INVOKED;
	IOPC_ASSERT(obj.oid == oid);
	const SerialIdType serialId = DbTypeManager::getSerialId(conn, obj.oid);
	IOPC_FUNC_ENDED;
	return obj.serialId == serialId;
}

void PersIdentification::invalidate()
{
	valid = false;
}

void PersIdentification::setValid()
{
	valid = true;
}


}
