/**
 * @file object.cpp
 *
 * @ingroup iopcmeta
 *
 * @author Petr Cermak
 */

#include "object.h"
#include "typeManager.h"
#include "enhancedTypeBase.h"
#include <sstream>
#include <iostream>

using namespace std;

namespace iopc {

Object::Object()
{
	metaInitialized = false;
	dirtyInitialized = false;
	dirty = false;
}

// We have to do late initialization as the typeid of the current object
// isn't available in the constructor.

void Object::initMeta() const
{
	lock.lock();
	if (metaInitialized) {
		lock.unlock();
		return;
	}
	IOPC_FUNC_INVOKED;

	type = &getTypeInternal();

	// Fill attributes for this class
	type->fillAttributeValues(*this, attributes);

	// Fill attributes for all parents
	const Type::AllParentsList& parents = type->getAllParents();
	for (Type::AllParentsIterator it = parents.begin(); it != parents.end(); it++) {
//		if ((*it)->isTypeOf(TypeDesc<Object>::getType())) {
			(*it)->fillAttributeValues(*this, attributes);

	}



	metaInitialized = true;
	lock.unlock();
	IOPC_FUNC_ENDED;
}

const Type& Object::getType() const
{
	initMeta();
	return *type;
}

const Type& Object::getTypeInternal() const
{
	return TypeManager::getTypeByTypeInfo(&typeid(*this));
}

const AttributeValue& Object::getAttributeValue(const std::string& name) const
{
	initMeta();

	map<string, AttributeValue>::iterator it = attributes.find(name);
	if(it == attributes.end()) {
		IOPC_LOG_THROW(IopcIncorrectUsageException(string("No ") + name + " attribute in the " + getType().getQualifiedName() + " class."));
	}
	return (*it).second;
}

void Object::initDirty()
{
	lock.lock();
	if (dirtyInitialized) {
		lock.unlock();
		return;
	}
	IOPC_FUNC_INVOKED;
	initMeta();

	for(map<string, AttributeValue>::const_iterator it = attributes.begin(); it != attributes.end(); it++) {
		if ((*it).second.getAttribute().getType().getDataTypeClass() == Type::IOPC_TYPECLASS_ENHANCED) {
			((EnhancedTypeBase*)(*it).second.getValue())->setDirtyPtr(&dirty);
		}
	}

	dirtyInitialized = true;
	lock.unlock();
	IOPC_FUNC_INVOKED;
}

bool Object::isDirty()
{
	IOPC_FUNC_INVOKED;
	initDirty();
	return dirty;
}

void Object::resetDirty()
{
	initDirty();
	dirty = false;
}

void Object::setDirty(bool dirty)
{
	initDirty();
	this->dirty = dirty;
}

string Object::toString() const
{
	ostringstream str;
	str << getType().getQualifiedName() << "@" << reinterpret_cast<uint> (this);
	return str.str();
}

int Object::compare(const Object& o) const
{
	//IOPC_LOG_THROW(IopcNotSupportedException("Compare not supported on this type."));

	if (this < &o) {
		return -1;
	} else if (this > &o) {
		return 1;
	} else {
		return 0;
	}
}

Object* Object::clone() const
{
	Object* copy = new Object();
	return copy;
}

int Object::getMemCost() const
{
	int sum = 0;
	initMeta();

	for (AttributeValuesIterator it = attributes.begin(); it != attributes.end(); it++)
	{
		const AttributeValue& attrVal = it->second;
		sum += attrVal.getAttribute().getType().getSizeOf(attrVal.getValue());
	}
	return sum;
}

}
