/**
 * @file main.cpp
 *
 * @author Petr Cermak
 *
 * @brief Handles command line arguments and manages the translation process
 */

#include <string>
#include <fstream>
#include <iostream>

#include "iopccommon/trace.h"
#include "iopccommon/util.h"
#include "iopccommon/iopcCommon.h"
#include "iopccommon/defs.h"
#include "xmlSupport.h"
#include "classGenerator.h"
#include <xercesc/parsers/XercesDOMParser.hpp>
#include <xercesc/util/PlatformUtils.hpp>
#include <xercesc/dom/DOM.hpp>
#include <xercesc/sax/HandlerBase.hpp>


using namespace std;
using namespace iopc;
XERCES_CPP_NAMESPACE_USE;

int main(int argc, char *argv[])
{
	IopcCommon::init(LogWriter::WARNING | LogWriter::ERROR | LogWriter::INFO, 0);

	if (argc != 3) {
		LogWriter::logError("IOPC SP version 2");
		LogWriter::logError("Generates type descriptions from GCCXML output");
		LogWriter::logError("Processes descendants of the iopc::Object class");
		LogWriter::logError("");
		LogWriter::logError("Usage: iopcsp gccxml_output.xml filename.cpp");
		LogWriter::logError("Creates filename.ih and filename.ic files that should be compiled and linked together with the user application");
		return 1;
	}

	string ofnameh = string(argv[2]) + ".ih";
	string ofnamec = string(argv[2]) + ".ic";

	LogWriter::logInfo("Init Xerces...");
	try {
    	XMLPlatformUtils::Initialize();
	}
	catch (const XMLException& exc) {
		LogWriter::logError(StringUtil::join("Exception in XMLPlatformUtils::Initialize: ", TOSTRING(exc.getMessage())));
		return 1;
	}

	LogWriter::logInfo("Xerces parser creation...");
	XercesDOMParser* parser = new ::XercesDOMParser();
	ErrorHandler* errHandler = (ErrorHandler*) new HandlerBase();
	parser->setErrorHandler(errHandler);
	parser->setDoNamespaces(false);
	parser->setDoSchema(false);
	LogWriter::logInfo("Xerces parsing...");
	try {
		parser->parse(argv[1]);
	} catch (const XMLException& exc) {
		LogWriter::logError(StringUtil::join("Parse error: ", TOSTRING(exc.getMessage())));
		return 1;
	} catch (const DOMException& exc) {
		LogWriter::logError(StringUtil::join("Parse error: ", TOSTRING(exc.getMessage())));
		return 1;
	} catch (...) {
		LogWriter::logError("Unexpected error");
		return 1;
	}
	LogWriter::logInfo("Processing DOM...");

	string classFileName = StringUtil::removeFileExt(argv[1]);
	ClassGenerator* cg = new ClassGenerator(parser->getDocument(), classFileName);
	cg->init();
	cg->getClasses();

	ofstream outh(ofnameh.c_str());
	ofstream outc(ofnamec.c_str());
	if (!outh || !outc) {
		LogWriter::logError("Could not open output file");
		return 1;
	}

	bool success = cg->writeHeader(outh);
	if (success) {
		#ifdef WIN32
		const string delim = "\\";
		#else
		const string delim = "/";
		#endif
		unsigned int occur = ofnameh.rfind(delim, ofnameh.size());
		if (occur != string::npos) {
			ofnameh = ofnameh.substr(occur + 1);
		}

		cg->writeCpp(outc, ofnameh);
	}
	//StringUtil::splitString(string(argv[1]), ".", classFileName);


	LogWriter::logInfo("Terminate Xerces...");
	delete cg;
	delete parser;
	delete errHandler;

	XMLPlatformUtils::Terminate();
	IopcCommon::shutdown();
	//cin.getline(NULL, 0);
	return 0;
}

