#include <string>
#include <iostream>
#include <vector>

#include "schema.h"
//#include "iopclib/cache/voidCache.h"
#include "iopclib/cache/arcCache.h"
#include "iopclib/cache/lruCache.h"
//#include "iopclib/cache/arcCacheST.h"
//#include "iopclib/cache/lruCacheST.h"

using namespace std;
using namespace iopc;

void setupDb(Connection* conn)
{
	// drop script
	// script execution can throw exceptions as the dropped structures may not be present in the database
	vector<string> script;
	script = ScriptsGenerator::getDbDropScript(conn->getDriver());
	for(vector<string>::const_iterator it = script.begin(); it != script.end(); it++) {
		try {
			conn->sqlNonQuery(*it);
		} catch (IopcException& e) {
			cout << e.what() << endl;
		}
	}

	// create script
	// If the script fails, some of the tables couldn't be dropped and recreated because of dependencies
	// created by other examples.
	// You have to resolve the dependencies and drop these tables from SQL*Plus manually:
	// select name, referenced_name from user_dependencies where referenced_name='objectname';
	script = ScriptsGenerator::getDbCreateScript(conn->getDriver());
	for(vector<string>::const_iterator it = script.begin();
		it != script.end(); it++) {
		conn->sqlNonQuery(*it);
	}

}

int main(int argc, char *argv[])
{
	CachedDatabase* db = NULL;
	CachedConnection* conn = NULL;
	try {
		// Uncomment the INFO_SQL mask to see what SQL statements are sent to the database
		Iopc::init(new TextFileMetadataLoader("metadata.config"), LogWriter::ERROR | LogWriter::WARNING | LogWriter::INFO_SQL, LogWriter::NONE);

		// Create a cached connection
		db = new CachedDatabase(
				DriverManager::getDriver("IopcOracle10g").getDatabase("XE"),
				new ComposedCache()
			);
		conn = (CachedConnection*) db->getConnection("username=iopc;password=iopc");

		// Set up cache keeper. You can try other cache types and other strategies
		CacheKeeper cacheKeeper(-15000, 20, 40);
		cacheKeeper.setCache(conn, new HashLruCache());
		cacheKeeper.setCache(conn, TypeDesc<Employee>::getType(), new HashArcCache());
		cacheKeeper.setStrategy(conn, Strategy::lazy);
		cacheKeeper.runMaintainer();

		conn->open();

		// Create the database schema
		setupDb(conn);

		// Insert some objects
		DbPtr<Person> person;
		person->name = "Mary Major";
		person->age = 60;
		person.bePersistent(conn);

		DbPtr<Employee> employee;
		employee->name = "Ola Nordmann";
		employee->age = 45;
		employee->salary = 60000;
		employee.bePersistent(conn);

		DbPtr<Student> student;
		student->name = "Richard Doe";
		student->age = 22;
		student->studcardid = "WCD-3223";
		student->supervisor = employee;
		student.bePersistent(conn);

		DbPtr<PhdStudent> phd;
		phd->name = "Joe Bloggs";
		phd->age = 27;
		phd->studcardid = "PHD-1234";
		phd->scholarship = 12000;
		phd->supervisor = employee;
		phd.bePersistent(conn);
		conn->commit();

		// CachePointer - lock the object in the cache
		{
			CachePtr<PhdStudent> phdPtr = *phd;
			// Update the PhdStudent
			phdPtr->scholarship = 13000;
			phdPtr->supervisor = employee;
		}

		// List persons older than 30
		SimpleQuery q("$::Person::age$ >= 30");
		cout << endl << "Persons older than 30" << endl;
		Result<Person> r(conn, q);
		r.open();
		for (Result<Person>::iterator it = r.begin(); it != r.end(); it++) {
			cout << "name: " << it->name << " age: " << it->age << endl;
		}
		r.close();

		// Find Persons older than 30
		// This time we will select only from the simple view
		// -> only Person instances are returned
		FreeQuery q2("$::Person[db.view.sv]$ WHERE $::Person[db.view.sv]::age$ >= 30");
		cout << endl << "Persons (only) older than 30 " << endl;
		Result<Person> r2(conn, q2);
		r2.open();
		for (Result<Person>::iterator it = r2.begin(); it != r2.end(); it++) {
			cout << "name: " << it->name << " age: " << it->age << endl;
		}
		r2.close();


		cacheKeeper.stopMaintainer();
		conn->close();
		db->returnConnection(conn);
		DriverManager::getDriver("IopcOracle10g").returnDatabase(db);

		conn = NULL;
		db = NULL;


		Iopc::shutdown();
	} catch (IopcException& e) {
		cout << e.what() << endl;
	}

	if (db && conn) conn->close();
	if (db) DriverManager::returnDatabase(db);

	cout << "-----------" << endl;
	cout << "Press enter" << endl;
	cin.getline(NULL, 0);
	return 0;
}


