#include <string>
#include <iostream>
#include <vector>

#include "schema.h"

using namespace std;
using namespace iopc;

int main(int argc, char *argv[])
{
	try {
		IopcMeta::init(LogWriter::ERROR | LogWriter::WARNING, LogWriter::NONE);

		// List registered types
		const map<string, const Type*>& types = TypeManager::getTypesMap();
		cout << "Registered types:" << endl;
		for(
		  map<string, const Type*>::const_iterator it = types.begin();
		  it != types.end(); it++) {
			cout << (*it).first << endl;
		}

		// Obtain the Type object reference
		const Type& t = TypeManager::getType("::Person");
		//const Type& t = TypeDesc<Person>::getType()

		// List parents
		cout << endl << "Parents of the " << t.getQualifiedName() << " class" << endl;;
		const Type::ParentsList& parents = t.getParents();
		for(Type::ParentsIterator it = parents.begin();	it != parents.end(); it++) {
			cout << (*it)->getQualifiedName() << endl;
		}
		// Shortcut for the first parent
		// cout << t.getFirstParent().getQualifiedName() << endl;

		// List children
		cout << endl << "Children of the " << t.getQualifiedName() << " class" << endl;;
		const Type::ChildrenList& children = t.getChildren();
		for(Type::ChildrenIterator it = children.begin(); it != children.end(); it++) {
			cout << (*it)->getQualifiedName() << endl;
		}

		// List attributes
		const Type::AttributesMap& attributes = t.getAttributes();
		cout << endl << "Its attributes" << endl;
		for(Type::AttributesIterator it = attributes.begin(); it != attributes.end(); it++) {
			const Attribute& attr = it->second;
			switch (attr.getVisibility()) {
				case Attribute::VISIBILITY_PUBLIC: cout << "public "; break;
				case Attribute::VISIBILITY_PRIVATE: cout << "private "; break;
				case Attribute::VISIBILITY_PROTECTED: cout << "protected ";break;
			}
			cout << it->second.getType().getQualifiedName() 	<< " " << it->first << endl;
		}

		// Create new Employee instance
		Employee* e = dynamic_cast<Employee*>(TypeManager::getType("::Employee").newObjectInstance());
		// Set its values
		*((EString*)e->getAttributeValue("name").getValue()) = "Ola Nordmann";
		*((EShort*)e->getAttributeValue("age").getValue()) = 45;
		*((EInt*)e->getAttributeValue("salary").getValue()) = 60000;
		cout << endl << "Dynamically created Employee instance. " << endl;
		cout << "Following line should be: Ola Nordmann 45 60000" << endl;
		cout << e->name << " " << e->age << " " << e->getSalary() << endl;

		// List the attribute values
		cout << endl << "And once more: " << endl;
		const Object::AttributeValuesMap& attrVals = e->getAttributeValues();
		for(Object::AttributeValuesIterator it = attrVals.begin(); it != attrVals.end(); it++) {
			const AttributeValue& val = it->second;
			cout << it->first << ": ";
			if (val.getAttribute().getType().getDataTypeClass() == Type::IOPC_TYPECLASS_ENHANCED) {
				cout << ((EnhancedTypeBase*)val.getValue())->toString() << endl;
			}
		}

		// Type comparison
		cout << endl << "t1 is Person, t2 is Student" << endl;
		const Type& t1 = TypeDesc<Person>::getType();
		const Type& t2 = TypeDesc<Student>::getType();
		if (t1 == t2) { cout << "t1 is t2" << endl; }
		if (t1 != t2) { cout << "t1 is not t2" << endl; }
		if (t1.isTypeOf(t2)) { cout << "t1 is t2 or t1 is descendant of t2" << endl; }
		if (t2.isTypeOf(t1)) { cout << "t1 is t2 or t2 is descendant of t1" << endl; }
		if (t1.is<Student>()) { cout << "t1 is of type Student" << endl; }
		if (t2.is<Student>()) { cout << "t2 is of type Student" << endl; }
		if (t1.isTypeOf<Person>()) { cout << "t1 is of type Person or its descendant" << endl; }
		if (t2.isTypeOf<Person>()) { cout << "t2 is of type Person or its descendant" << endl; }


		IopcMeta::shutdown();
	} catch (IopcException& e) {
		cout << e.what();
	}
	cout << "-----------" << endl;
	cout << "Press enter" << endl;
	cin.getline(NULL, 0);
	return 0;
}


