/**
 * @file basicConnection.cpp
 *
 * @author Michal Kopecky
 * @author Josef Troch Josef Troch (<a href="mailto:josef.troch@email.cz">josef.troch@email.cz</a>)
 * @author Jan Hadrava
 * @author Petr Cermak
 *
 * @ingroup iopcdb
 */


#include "iopccommon/trace.h"
#include "database.h"
#include "basicConnection.h"
#include <string>
#include "connectionImpl.h"

using namespace std;

namespace iopc {

// static variable
BasicConnection::CursorSet BasicConnection::allCursors;
Lock BasicConnection::allLock;

BasicConnection::BasicConnection(ConnectionImpl* implementation, BasicDatabase& database)
: database(database)

{
	this->implementation = implementation;
	autoCommit = false;
	opened = false;
}

BasicConnection::~BasicConnection()
{
	IOPC_FUNC_INVOKED;
	if (isOpened()) {
		LogWriter::logWarning("Deleting connected Oracle10gConnectionImpl. Disconnecting...");
		try {
			disconnect(); // calls doCommit() and doClose()
		} catch (const IopcException& e) {
			LogWriter::logError(string("Error disconnecting db session: ") + e.what());
		}
	}
	if (cursors.size() > 0) {
		LogWriter::logWarning("Deleting Oracle10gCursors with cursors!");
		closeAllCursors();
	}
	IOPC_FUNC_ENDED;
}

Connection* BasicConnection::getBasicConnection()
{
	return this;
}


void BasicConnection::addCursor(Cursor* cur)
{
	allLock.lock();
	try {
		allCursors.insert(cur);
		cursors.insert(cur);
	}
	IOPC_LOG_CATCH(...)
		allLock.unlock();
		IOPC_LOG_RETHROW(...);
	IOPC_LOG_CAUGHT
	allLock.unlock();
}

void BasicConnection::removeCursor(Cursor* cur)
{
	allLock.lock();
	try {
		CursorSetIterator iter = cursors.find(cur);
		if (iter != cursors.end())
			cursors.erase(iter);
		CursorSetIterator iter2 = allCursors.find(cur);
		if (iter2 != allCursors.end()) {
			allCursors.erase(iter2);
		}
	}
	IOPC_LOG_CATCH(...)
		allLock.unlock();
		IOPC_LOG_RETHROW(...);
	IOPC_LOG_CAUGHT
	allLock.unlock();
}

void BasicConnection::open()
{
	IOPC_FUNC_INVOKED;
	lock.lock();
	if (opened) {
		lock.unlock();
		IOPC_LOG_THROW(IopcDbException("Connection is already opened"));
	}
	try {
		implementation->open();
	}
	IOPC_LOG_CATCH(...)
		lock.unlock();
		IOPC_LOG_RETHROW(...);
	IOPC_LOG_CAUGHT
	opened = true;
	lock.unlock();
	IOPC_FUNC_ENDED;
}

void BasicConnection::close()
{
	IOPC_FUNC_INVOKED;
	lock.lock();
	allLock.lock();
	try {
		if (cursors.size() > 0) {
			closeAllCursors();
		}
	}
	IOPC_LOG_CATCH(...)
		allLock.unlock();
		lock.unlock();
		IOPC_LOG_RETHROW(...);
	IOPC_LOG_CAUGHT
	allLock.unlock();

	try {
		if (opened) {
			implementation->close();
		}
	}
	IOPC_LOG_CATCH(...)
		lock.unlock();
		IOPC_LOG_RETHROW(...);
	IOPC_LOG_CAUGHT

	opened = false;
	lock.unlock();
	IOPC_FUNC_ENDED;
}

bool BasicConnection::isOpened()
{
	return opened;
}

Cursor* BasicConnection::sql(const string& sqlCommand)
{
	lock.lock();
	if (!opened) {
		lock.unlock();
		IOPC_LOG_THROW(IopcDbException("Connection must be opened first"));
	}
	Cursor* retval = NULL;
	try {
		retval = new Cursor(implementation->getCursor(sqlCommand), sqlCommand);
		addCursor(retval);
	}
	IOPC_LOG_CATCH(...)
		lock.unlock();
		IOPC_LOG_RETHROW(...);
	IOPC_LOG_CAUGHT
	lock.unlock();
	return retval;
}

void BasicConnection::returnCursor(Cursor* cur)
{
	IOPC_FUNC_INVOKED;
	lock.lock();
	if (!opened) {
		lock.unlock();
		IOPC_LOG_THROW(IopcDbException("Connection must be opened first"));
	}
	try {
		if (cur->isOpened()) {
			cur->close();
		}
		removeCursor(cur);
		implementation->returnCursor(cur->getImplementation());
		delete cur;
	}
	IOPC_LOG_CATCH(...)
		lock.unlock();
		IOPC_LOG_RETHROW(...);
	IOPC_LOG_CAUGHT
	lock.unlock();
}

void BasicConnection::closeAllCursors()
{
	IOPC_FUNC_INVOKED;
	allLock.lock();
	try {
		for (CursorSetIterator iter = cursors.begin(); iter != cursors.end(); ++iter) {
			(*iter)->close();
			allCursors.erase(*iter);
			implementation->returnCursor((*iter)->getImplementation());
			delete (*iter);
		}
		cursors.clear();
	}
	IOPC_LOG_CATCH(...)
		allLock.unlock();
		IOPC_LOG_RETHROW(...);
	IOPC_LOG_CAUGHT
	allLock.unlock();
	IOPC_FUNC_ENDED;
}


void BasicConnection::sqlNonQuery(const std::string& name)
{
	IOPC_FUNC_INVOKED;
	Cursor* cur = sql(name);

	try {
		cur->open();
		cur->prepare();
		cur->execute();
		cur->close();
	}
	catch (...) {
		returnCursor(cur);
		throw ;
	}
	returnCursor(cur);
	IOPC_FUNC_ENDED;
}

void BasicConnection::commit()
//Commit database, after storing modified objects
{
	if (!opened) {
		IOPC_LOG_THROW(IopcDbException("Connection must be opened first"));
	}

	implementation->commit();
}

void BasicConnection::rollback()
//Rollback database, then destroy all copies in memory
{
	if (!opened) {
		IOPC_LOG_THROW(IopcDbException("Connection must be opened first"));
	}

	implementation->rollback();
}

void BasicConnection::savepoint(const string& name)
//Create savepoint, after storing all modified objects
{
	if (!opened) {
		IOPC_LOG_THROW(IopcDbException("Connection must be opened first"));
	}

	implementation->savepoint(name);
}

void BasicConnection::rollbackToSavepoint(const string& name)
//Rollback to savepoint, then destroy all data in memory
{
	implementation->rollbackToSavepoint(name);
}

void BasicConnection::disconnect()
//Commit work and disconnect from database
{
	IOPC_FUNC_INVOKED;
	lock.lock();
	try {
		if (opened) {
			commit();
			close();
		}
	}
	IOPC_LOG_CATCH(...)
		lock.unlock();
		IOPC_LOG_RETHROW(...);
	IOPC_LOG_CAUGHT
	lock.unlock();
/*	database = NULL;
	userName.clear();
	password.clear();*/

	IOPC_FUNC_ENDED;
}

void BasicConnection::abort()
//Rollback work and disconnect from database
{
	IOPC_FUNC_INVOKED;
	lock.lock();
	try {
		if (opened) {
			rollback();
			close();
		}
	}
	IOPC_LOG_CATCH(...)
		lock.unlock();
		IOPC_LOG_RETHROW(...);
	IOPC_LOG_CAUGHT
	lock.unlock();
/*	database = NULL;
	userName.clear();
	password.clear();*/

	IOPC_FUNC_ENDED;
}

void BasicConnection::setAutoCommit(bool ac)
{
	implementation->setAutoCommit(ac);
	autoCommit = ac;
}

Connection& BasicConnection::operator<<(const string& sqlCommand)
{
	sqlNonQuery(sqlCommand);

	return *this;
}


bool BasicConnection::operator==(const Connection& dbConn) const
//Tests, if the Database connections are the same
{
	return this == &dbConn;
}

bool BasicConnection::operator!=(const Connection& dbConn) const
//Tests, if the Database connections are not the same
{
	return this != &dbConn;
}

}/*namespace iopc*/
