/**
 * @file connectionImpl.h
 *
 * @author Petr Cermak
 *
 * @ingroup iopcdb
 */

#ifndef CONNECTIONIMPL_H_
#define CONNECTIONIMPL_H_

#include "cursorImpl.h"
#include "iopccommon/defs.h"

namespace iopc {

/**
 * @brief Interface of driver-side part of the @a Connection class implementation
 *
 * Each database driver must provide a class that implements this interface
 *
 * Instances of this class are obtained using the @a DatabaseImpl::getConnection() method
 * and are returned using the @a DatabaseImpl::returnConnection() method.
 *
 * @ingroup iopcdb
 */
class DLL_EXTERNAL ConnectionImpl {
public:

	/**
	 * @brief Commits current transaction
	 */
	virtual void commit() = 0;

	/**
	 * @brief Rolls back current transaction
	 */
	virtual void rollback() = 0;

	/**
	 * @brief Creates a savepoint
	 *
	 * @param name Savepoint name
	 */
	virtual void savepoint(const std::string& name) = 0;

	/**
	 * @brief Rolls back to the specified savepoint
	 *
	 * @param name Name of the savepoint to which to roll back to.
	 */
	virtual void rollbackToSavepoint(const std::string& name) = 0;

	/**
	 * @brief Opens the connection
	 *
	 * This call should open a physical link to the DBMS
	 */
	virtual void open() = 0;

	/**
	 * @brief Closes the connection
	 *
	 * This call should close the physical link to the DBMS
	 */
	virtual void close() = 0;

	/**
	 * @brief Changes the autocommit state set on this connection
	 */
	virtual void setAutoCommit(bool ac) = 0;

	/**
	 * @brief Executes a SQL statement and returns a @a CursorImpl object which is responsible
	 * for the statement execution, parameter binding and query results retrieval.
	 *
	 * @param sqlCommand The SQL statement
	 *
	 * @returns Implementation of the cursor
	 */
	virtual CursorImpl* getCursor(const std::string& sqlCommand) = 0;

	/**
	 * @brief Cursors that are no longer needed are returned to the driver using this method.
	 *
	 * Database driver should release all allocated resources associated with the cursor.
	 *
	 * @param cur Implementation of the cursor
	 */
	virtual void returnCursor(CursorImpl* cur) = 0;

	virtual ~ConnectionImpl() {};
};

}

#endif /*CONNECTIONIMPL_H_*/
