/**
 * @file cursor.cpp
 *
 * @author Josef Troch Josef Troch (<a href="mailto:josef.troch@email.cz">josef.troch@email.cz</a>)
 * @author Jan Hadrava
 * @author Petr Cermak
 */

#include "cursor.h"
#include "iopccommon/trace.h"
#include "cursorImpl.h"
#include "iopcmeta/typeDesc.h"
#include "iopcmeta/enhancedString.h"
#include <string>

using namespace std;

namespace iopc {

Cursor::Cursor(CursorImpl* cursorImpl, const string& sql)
{
	state = CLOSED;
	this->implementation = cursorImpl;
	this->sql = sql;
}


Cursor::~Cursor()
{
	IOPC_FUNC_INVOKED;

	for (ParametersIterator it = inParams.begin(); it != inParams.end(); it++) {
		delete it->second;
	}

	for (ParametersIterator it = outParams.begin(); it != outParams.end(); it++) {
		delete it->second;
	}

	if (state != CLOSED) {
		LogWriter::logWarning("Destructor on nonclosed cursor called, closing.");
		close();
	}
}

void Cursor::prepareState()
{
	switch (state)
	{
		case CLOSED:
			open();
		case OPENED:
		case EXECUTED:
			prepare();
		default:
			break;
	}
}

void Cursor::open()
{
	IOPC_FUNC_INVOKED;
	if (state != CLOSED) {
		IOPC_LOG_THROW(IopcDbException("Cursor must be closed before opening"));
	}
	implementation->open();
	state = OPENED;
	IOPC_FUNC_ENDED;
}

void Cursor::close()
{
	IOPC_FUNC_INVOKED;
	if (isOpened()) {
		implementation->close();
		state = CLOSED;
	}
	IOPC_FUNC_ENDED;
}

void Cursor::prepare()
{
	IOPC_FUNC_INVOKED;
	if (state == CLOSED) {
		open();
	}
	implementation->prepare();
	state = PREPARED;
	IOPC_FUNC_ENDED;
}

void Cursor::preExecBindNull(const int pos, const Type& varType, const MetadataHolder& attributes)
{
	IOPC_FUNC_INVOKED;
	if (state != PREPARED) {
		IOPC_LOG_THROW(IopcDbException("Cursor must be in sate PREPARED before calling preExecBindNull. Call prepare first."));
	}
	implementation->preExecBind(pos, NULL, 0, varType, true, attributes);
	IOPC_FUNC_ENDED;
}

void Cursor::preExecBind(const int pos, const void* varAddr, const unsigned int size, const Type& varType, const MetadataHolder& attributes)
{
	IOPC_FUNC_INVOKED;
	if (state != PREPARED) {
		IOPC_LOG_THROW(IopcDbException("Cursor must be in sate PREPARED before calling preExecBind. Call prepare first."));
	}

	const MetadataHolder& meta = (attributes == MetadataHolder::emptyMeta) ? varType : attributes;

	implementation->preExecBind(pos, varAddr, size, varType, false, meta);

	IOPC_FUNC_ENDED;
}

void Cursor::execute()
{
	IOPC_FUNC_INVOKED;
	prepareState();
	for (ParametersIterator it = inParams.begin(); it != inParams.end(); it++)
	{
		it->second->preExec(this);
	}
	LogWriter::logSql(sql);
	implementation->execute();
	state = EXECUTED;
	for (ParametersIterator it = inParams.begin(); it != inParams.end(); it++)
	{
		it->second->postExec(this);
	}
	fetched = false;
	/*for (ParametersIterator it = outParams.begin(); it != outParams.end(); it++)
	{
		it->second->preFetch(this);
	}*/
	IOPC_FUNC_ENDED;
}


void Cursor::preFetchBind(const int pos, const void* varAddr, const unsigned int size, const Type& varType, bool* isNull, const MetadataHolder& attributes)
{
	IOPC_FUNC_INVOKED;
	if (state != EXECUTED) {
		IOPC_LOG_THROW(IopcDbException("Cursor must be in sate EXECUTED before calling preFetchBind. Call execute first."));
	}

	const MetadataHolder& meta = (attributes == MetadataHolder::emptyMeta) ? varType : attributes;

	if (isNull == NULL)
	{
		bool tmpIsNull;
		implementation->preFetchBind(pos, varAddr, size, varType, &tmpIsNull, meta);
	} else {
		implementation->preFetchBind(pos, varAddr, size, varType, isNull, meta);
	}
	IOPC_FUNC_ENDED;
}

bool Cursor::fetchNext()
{
	IOPC_FUNC_INVOKED;
	if (state != EXECUTED) {
		IOPC_LOG_THROW(IopcDbException("Cursor must be in sate EXECUTED before calling fetchNext. Call execute first."));
	}
	if (!fetched) {
		for (ParametersIterator it = outParams.begin(); it != outParams.end(); it++)
		{
			it->second->preFetch(this);
		}
		fetched = true;
	}

	bool ret = implementation->fetchNext();
	if (ret) {
		for (ParametersIterator it = outParams.begin(); it != outParams.end(); it++)
		{
			it->second->postFetch(this);
		}
	}
	IOPC_FUNC_ENDED;
	return ret;
}

bool Cursor::fetchPrev()
{
	IOPC_FUNC_INVOKED;
	if (state != EXECUTED) {
		IOPC_LOG_THROW(IopcDbException("Cursor must be in sate EXECUTED before calling fetchPrev. Call execute first."));
	}
	bool ret = implementation->fetchPrev();
	for (ParametersIterator it = inParams.begin(); it != inParams.end(); it++)
	{
		it->second->postFetch(this);
	}
	IOPC_FUNC_ENDED;
	return ret;
}

long Cursor::getPosition()
{
	IOPC_FUNC_INVOKED;
	return implementation->getPosition();
}




void Cursor::checkAndDeleteParam(const int pos, ParametersMap& map)
{
	ParametersIterator it = map.find(pos);
	if (it != map.end()) {
		delete it->second;
	}
	fetched = false;
}

void Cursor::addNullInParam(const int pos, const Type& type, const MetadataHolder& attributes)
{
	checkAndDeleteParam(pos, inParams);
	inParams[pos] = new NullParameter(Parameter::IN, pos, type, attributes);
}


void Cursor::addInParam(const int pos, const void* varAddr, const Type& varType, const MetadataHolder& attributes)
{
	addParam(Parameter::IN, pos, varAddr, varType, attributes);
}

void Cursor::addEnhancedInParam(const int pos, EnhancedTypeBase& obj, const MetadataHolder& attributes)
{
	addEnhancedParam(Parameter::IN, pos, obj, attributes);
}

void Cursor::addOutParam(const int pos, const void* varAddr, const Type& varType, const MetadataHolder& attributes)
{
	addParam(Parameter::OUT, pos, varAddr, varType, attributes);
}

void Cursor::addEnhancedOutParam(const int pos, EnhancedTypeBase& obj, const MetadataHolder& attributes)
{
	addEnhancedParam(Parameter::OUT, pos, obj, attributes);
}

void Cursor::addParam(Parameter::ParamType direction, const int pos, const void* varAddr, const Type& varType, const MetadataHolder& attributes)
{
	ParametersMap& params = (direction == Parameter::IN) ? inParams : outParams;

	switch (varType.getDataTypeClass())
	{
		case Type::IOPC_TYPECLASS_ENHANCED:
			checkAndDeleteParam(pos, params);
			params[pos] = new EnhancedTypeParameter(direction, pos, *(const_cast<EnhancedTypeBase*>(dynamic_cast<const EnhancedTypeBase*>((const Object*)varAddr))), attributes);
			break;
		case Type::IOPC_TYPECLASS_STRING:
		case Type::IOPC_TYPECLASS_SIMPLE:
			checkAndDeleteParam(pos, params);
			params[pos] = new SimpleTypeParameter(direction, pos, varAddr, varType, attributes);
			break;
		default:
			IOPC_LOG_THROW(IopcIncorrectUsageException("Cursor doesn't know how to bind this typeclass"));
			break;
	}
}

void Cursor::addEnhancedParam(Parameter::ParamType direction, const int pos, EnhancedTypeBase& obj, const MetadataHolder& attributes)
{
	ParametersMap& params = (direction == Parameter::IN) ? inParams : outParams;
	checkAndDeleteParam(pos, params);
	params[pos] = new EnhancedTypeParameter(direction, pos, obj, attributes);
}


Parameter::Parameter(ParamType type, const int pos, const Type& varType, const MetadataHolder& attributes, bool setNull)
	: paramType(type), pos(pos), varType(varType), attributes(attributes), setNull(setNull), isNull(false)
{
}

void Parameter::preExec(Cursor* cur)
{
	if (paramType == IN) {
		doPreExec(cur);
		/*if (!setNull) {
			doPreExec(cur);
		} else {
			doPreExecNull(cur);
			//cur->preExecBindNull(pos, varType);
		}*/
	}
}

void Parameter::postExec(Cursor* cur)
{
	if (paramType == IN) {
		doPostExec(cur);
	}
}

void Parameter::preFetch(Cursor* cur)
{
	if (paramType == OUT) {
		doPreFetch(cur);
	}
}

void Parameter::postFetch(Cursor* cur)
{
	if (paramType == OUT) {
		doPostFetch(cur);
	}
}


SimpleTypeParameter::SimpleTypeParameter(ParamType type, const int pos, const void* varAddr, const Type& varType, const MetadataHolder& attributes)
	: Parameter(type, pos, varType, attributes, false), varAddr(varAddr)
{
	if (varType.getDataTypeClass() == Type::IOPC_TYPECLASS_STRING) {

		maxLen = 0;
		if (attributes.hasMeta("db.type.length")) {
			maxLen = attributes.getMeta("db.type.length").getIntValue();
		} else if (varType.hasMeta("db.type.length")){
			maxLen = varType.getMeta("db.type.length").getIntValue();
		} else {
			IOPC_LOG_THROW(IopcMetadataException("db.type.length", varType.getQualifiedName(), "Metadata expected."));
		}
	}
}

void SimpleTypeParameter::doPreExec(Cursor* cur)
{
	if (setNull) {
		cur->preExecBindNull(pos, varType);
	} else {
		if (varType.getDataTypeClass() == Type::IOPC_TYPECLASS_STRING) {
			void* bufPtr;
			unsigned int size;
			if (varType.getDataType() == Type::IOPC_TYPE_STRING) {
				tmpStr = *((std::string*)varAddr);
				bufPtr = tmpStr.externalizeCopy(maxLen);
				size = tmpStr.getBufferSize();
			} else { //if (varType.getDataType() == Type::IOPC_TYPE_WSTRING) {
				tmpWStr = *((std::wstring*)varAddr);
				bufPtr = tmpWStr.externalizeCopy(maxLen);
				size = tmpWStr.getBufferSize();
			}
			cur->preExecBind(pos, bufPtr, size, varType, attributes);
		} else {
			cur->preExecBind(pos, varAddr, varType.getSizeOf(varAddr), varType, attributes);
		}
	}
}

void SimpleTypeParameter::doPreFetch(Cursor* cur)
{
	isNull = false;
	if (varType.getDataTypeClass() == Type::IOPC_TYPECLASS_STRING) {
		void* bufPtr;
		unsigned int size;
		if (varType.getDataType() == Type::IOPC_TYPE_STRING) {
			bufPtr = tmpStr.externalizeEmptyBuffer(maxLen);
			size = tmpStr.getBufferSize();
		} else { // if (varType.getDataType() == Type::IOPC_TYPE_WSTRING) {
			bufPtr = tmpWStr.externalizeEmptyBuffer(maxLen);
			size = tmpWStr.getBufferSize();
		}

		cur->preFetchBind(pos, bufPtr, size, varType, false, attributes);
	} else {
		cur->preFetchBind(pos, varAddr, varType.getSizeOf(varAddr), varType, &isNull, attributes);
	}
}

void SimpleTypeParameter::doPostFetch(Cursor* cur)
{
	if (varType.getDataType() == Type::IOPC_TYPE_STRING) {
		tmpStr.updateFromExternalized();
		*((std::string*)varAddr) = tmpStr;
	} else if (varType.getDataType() == Type::IOPC_TYPE_WSTRING) {
		tmpWStr.updateFromExternalized();
		*((std::wstring*)varAddr) = tmpWStr;
	}
}

EnhancedTypeParameter::EnhancedTypeParameter(ParamType type, const int pos, EnhancedTypeBase& obj, const MetadataHolder& attributes)
	: Parameter(type, pos, obj.getType(), attributes, obj.isNull()), variable(obj)
{
	maxLen = 0;
	isNull = false;
	if (varType.is<EString>() || varType.is<EWString>()) {
		if (attributes.hasMeta("db.type.length")) {
			maxLen = attributes.getMeta("db.type.length").getIntValue();
		} else if (varType.hasMeta("db.type.length")){
			maxLen = varType.getMeta("db.type.length").getIntValue();
		} else {
			IOPC_LOG_THROW(IopcMetadataException("db.type.length", varType.getQualifiedName(), "Metadata expected."));
		}
	}
}

void EnhancedTypeParameter::doPreExec(Cursor* cur)
{
	const Type& tp = variable.getInnerType();
	IOPC_DEBUGLOG(std::string("bind: ") + variable.toString());
	if (setNull) {
		cur->preExecBindNull(pos, tp);
	} else {
		void* value = variable.externalizeCopy(maxLen);
		unsigned int size = variable.getBufferSize();

		cur->preExecBind(pos, value, size, tp, attributes);
	}
}

void EnhancedTypeParameter::doPreFetch(Cursor* cur)
{
	void* value = variable.externalizeEmptyBuffer(maxLen);
	unsigned int size = variable.getBufferSize();
	const Type& tp = variable.getInnerType();

	isNull = false;
	cur->preFetchBind(pos, value, size, tp, &isNull, attributes);
}

void EnhancedTypeParameter::doPostFetch(Cursor* cur)
{
	if (isNull) {
		variable.setNull();
	} else {
		variable.setNull(false);
		variable.updateFromExternalized();
	}
	IOPC_DEBUGLOG(std::string("bind: ") + variable.toString());
}


NullParameter::NullParameter(ParamType direction, const int pos, const Type& varType, const MetadataHolder& attributes)
	: Parameter(direction, pos, varType, attributes, false)
{
}

void NullParameter::doPreExec(Cursor* cur)
{

	if (varType.getDataTypeClass() == Type::IOPC_TYPECLASS_ENHANCED) {
		IOPC_DEBUGLOG(varType.getQualifiedName() + " " +  EnhancedTypeBase::getInnerTypeFor(varType).getQualifiedName());
		cur->preExecBindNull(pos, EnhancedTypeBase::getInnerTypeFor(varType), attributes);
	} else {
		cur->preExecBindNull(pos, varType, attributes);
	}
}


}
