/**
 * @file databaseImpl.h
 *
 * @author Petr Cermak
 *
 * @ingroup iopcdb
 */

#ifndef DATABASEIMPL_H_
#define DATABASEIMPL_H_

#include "iopccommon/defs.h"
#include "connectionImpl.h"

namespace iopc {


/**
 * @brief Interface of driver-side part of the @a Database class implementation
 *
 * Each database driver must provide a class that implements this interface
 *
 * Instances of this class are obtained from the driver using @a Driver::getDatabaseImpl() method
 * and are returned using the @a Driver::returnDatabaseImpl() method.
 *
 * @ingroup iopcdb
 */
class  DLL_EXTERNAL DatabaseImpl {
public:

	/**
	 * @brief This method is invoked when user requests a new database connection by
	 * calling @a Database::getConnection().
	 *
	 * The implementation should create and return a new @a ConnectionImpl object.
	 *
	 * @param connStr A connection string. Its format is defined by the database driver.
	 * Usually it identifies the database user/schema.
	 *
	 * @returns Implementation of the connection
	 *
	 * @see Database
	 * @see Connection
	 */
	virtual ConnectionImpl* getConnection(const std::string& connStr) = 0;


	/**
	 * @brief Connections that are no longer needed are returned to the driver using this method.
	 *
	 * Database driver should release all allocated resources associated with the connection.
	 * This method is invoked from @a Database::returnConnection()
	 *
	 * @param conn Implementation of the connection.
	 *
	 * @see Database
	 * @see Connection
	 */
	virtual void returnConnection(ConnectionImpl* conn) = 0;

	virtual ~DatabaseImpl() {};
};

}


#endif /*DATABASEIMPL_H_*/
