/**
 * @file composedCache.cpp
 *
 * @author Jan Hadrava
 * @date 2004
 * @ingroup iopclibcache
 */


#include <map>
#include "extendedCache.h"
#include "composedCache.h"
#include "iopccommon/exceptionBase.h"

namespace iopc {

ComposedCache::ComposedCache(CacheSelector* cacheSel, StrategySelector* stratSel)
: cacheSel(cacheSel), stratSel(stratSel)
{
}

ComposedCache::ComposedCache()
: cacheSel(NULL), stratSel(NULL)
{
}


ComposedCache::~ComposedCache()
{
	if (cacheSel != NULL) delete cacheSel;
	if (stratSel != NULL) delete stratSel;
}

void ComposedCache::setCacheSelector(CacheSelector* cacheSel)
{
	this->cacheSel = cacheSel;
}

void ComposedCache::setStrategySelector(StrategySelector* strategySel)
{
	this->stratSel = strategySel;
}

void ComposedCache::setConnection(Connection* conn)
{
	IOPC_FUNC_INVOKED;
	if (cacheSel == NULL) return;
	checkState();
	Cache::setConnection(conn);
	for (ExtendedCache* cache = cacheSel->firstCache(); cache != NULL; cache = cacheSel->nextCache()) {
		cache->setConnection(conn);
	}
	IOPC_FUNC_ENDED;
}

void ComposedCache::clear()
{
	if (cacheSel == NULL) return;
	checkState();
	for (ExtendedCache* cache = cacheSel->firstCache(); cache != NULL; cache = cacheSel->nextCache()) {
		cache->clear(*stratSel);
	}
}

CacheLock& ComposedCache::makePersistent(const PersIdentification& persId, DatabaseObject& objectPtr)
{
	checkState();
	return cacheSel->selectCache(persId)->makePersistent(*stratSel, persId, objectPtr);
}

CacheLock& ComposedCache::getCacheLock(const PersIdentification& persId, bool rdonly)
{
	checkState();
	return cacheSel->selectCache(persId)->getCacheLock(*stratSel, persId, rdonly);
}

void ComposedCache::dbDelete(const PersIdentification& persId)
{
	checkState();
	cacheSel->selectCache(persId)->dbDelete(*stratSel, persId);
}

void ComposedCache::cacheDelete(const PersIdentification& persId)
{
	checkState();
	cacheSel->selectCache(persId)->cacheDelete(*stratSel, persId);
}

void ComposedCache::updateDirty()
{
	checkState();
	for (ExtendedCache* cache = cacheSel->firstCache(); cache != NULL; cache = cacheSel->nextCache()) {
		cache->updateDirty(*stratSel);
	}
}

void ComposedCache::removeAll()
{
	checkState();
	for (ExtendedCache* cache = cacheSel->firstCache(); cache != NULL; cache = cacheSel->nextCache()) {
		cache->removeAll(*stratSel);
	}
}

void ComposedCache::checkState()
{
	if (cacheSel == NULL || stratSel == NULL)
	{
		IOPC_LOG_THROW(IopcInvalidStateException("Must set selectors first. Call the setStrategySelector and setCacheSelector."));
	}
}

Cache* ComposedCache::clone()
{
	return new ComposedCache(cacheSel, stratSel);
}

}/*namespace iopc*/
