/**
 * @file cachedConnection.cpp
 *
 * @ingroup iopclib
 *
 * @author Petr Cermak
 */

#include "cachedConnection.h"

namespace iopc
{

CachedConnection::CachedConnection(Connection* connection, Cache* cache, CachedDatabase& database)
: database(database)
{
	IOPC_FUNC_INVOKED;
	this->inner = connection;
	this->cache = cache;
	if (connection->isOpened()) {
		cache->setConnection(inner);
	}
}

CachedConnection::~CachedConnection()
{
	cacheLock.wlock();
	cache->setConnection(NULL);
	cacheLock.unlock();
	IOPC_FUNC_INVOKED;
	delete cache;
	delete inner;
}


void CachedConnection::stopCache()
{
	cacheLock.wlock();
	try {
		/* the cache must be emptied to make it possible to
		** safely change its settings */
		cache->setConnection(NULL);
	} catch (...) {
		cacheLock.unlock();
		throw;
	}
	/* keeping cacheLock locked (will be unlocked by RestartCache()) */
}

void CachedConnection::restartCache()
{
	if (cache != NULL) {
		cache->setConnection(inner);	/* mutual cooperation goes on */
	}
	cacheLock.unlock();	/* unlocking lock locked by StopCache() */
}


Cursor* CachedConnection::sqlNoCache(const std::string& name)
{
	return inner->sql(name);
}

Cursor* CachedConnection::sql(const std::string& name)
{
	cacheLock.rlock();
	try {
		cache->updateDirty();
	} catch (...) {
		cacheLock.unlock();
		throw ;
	}
	cacheLock.unlock();
	return inner->sql(name);
}

void CachedConnection::returnCursor(Cursor* cur)
{
	inner->returnCursor(cur);
}

void CachedConnection::closeAllCursors()
{
	inner->closeAllCursors();
}

void CachedConnection::sqlNonQuery(const std::string& name)
{
	// copy local changes to the db
	cacheLock.rlock();
	try {
		cache->updateDirty();
	} catch (...) {
		cacheLock.unlock();
		throw ;
	}
	cacheLock.unlock();
	inner->sqlNonQuery(name);
}

void CachedConnection::sqlNonQueryNoCache(const std::string& name)
{
	inner->sqlNonQuery(name);
}


void CachedConnection::commit()
{
	//Commit database, after storing modified objects
	cacheLock.rlock();
	try {
		cache->updateDirty();
	} catch (...) {
		cacheLock.unlock();
		throw ;
	}
	cacheLock.unlock();
	inner->commit();
}

void CachedConnection::rollback()
{
	cacheLock.rlock();
	try {
		cache->removeAll();
	} catch (...) {
		cacheLock.unlock();
		throw ;
	}
	cacheLock.unlock();
	inner->rollback();
}


void CachedConnection::savepoint(const std::string& name)
{
	cacheLock.rlock();
	try {
		cache->updateDirty();
	} catch (...) {
		cacheLock.unlock();
		throw ;
	}
	cacheLock.unlock();
	inner->savepoint(name);
}

void CachedConnection::rollbackToSavepoint(const std::string& name)
{
	cacheLock.rlock();
	try {
		cache->removeAll();
	} catch (...) {
		cacheLock.unlock();
		throw ;
	}
	cacheLock.unlock();
	inner->rollbackToSavepoint(name);
}

void CachedConnection::open()
{
	IOPC_FUNC_INVOKED;
	inner->open();
	cacheLock.rlock();
	try {
		cache->setConnection(inner);
	} catch (...) {
		cacheLock.unlock();
		throw ;
	}
	cacheLock.unlock();
	IOPC_FUNC_ENDED;
}

void CachedConnection::close()
{
	IOPC_FUNC_INVOKED;
	cacheLock.rlock();
	try {
		cache->setConnection(NULL);
	} catch (...) {
		cacheLock.unlock();
		throw ;
	}
	cacheLock.unlock();
	inner->close();
	IOPC_FUNC_ENDED;
}

void CachedConnection::disconnect()
{
	inner->disconnect();
}

void CachedConnection::abort()
{
	inner->abort();
}

void CachedConnection::setAutoCommit(bool ac)
{
	inner->setAutoCommit(ac);
}

bool CachedConnection::isOpened()
{
	return inner->isOpened();
}

Connection& CachedConnection::operator << (
	    const std::string& command
	)
{
	this->sqlNonQuery(command);
	return *this;
}

bool CachedConnection::operator== (const Connection& dbConn) const
{
	return this == &dbConn;
}

bool CachedConnection::operator!= (const Connection& dbConn) const
{
	return this != &dbConn;
}

CacheLock& CachedConnection::makePersistent(const PersIdentification& persId, DatabaseObject& objectPtr)
{
	CacheLock* cl = NULL;
	cacheLock.rlock();
	try {
		cl = &cache->makePersistent(persId, objectPtr);
	} catch (...) {
		cacheLock.unlock();
		throw ;
	}
	cacheLock.unlock();
	return *cl;
}

CacheLock& CachedConnection::getCacheLock(const PersIdentification& persId, bool rdonly)
{
	CacheLock* cl = NULL;
	cacheLock.rlock();
	try {
		cl = &cache->getCacheLock(persId, rdonly);
	} catch (...) {
		cacheLock.unlock();
		throw ;
	}
	cacheLock.unlock();
	return *cl;
}

void CachedConnection::dbDelete(const PersIdentification& persId)
{
	cacheLock.rlock();
	try {
		cache->dbDelete(persId);
	} catch (...) {
		cacheLock.unlock();
		throw ;
	}
	cacheLock.unlock();
}

void CachedConnection::cacheDelete(const PersIdentification& persId)
{
	cacheLock.rlock();
	try {
		cache->cacheDelete(persId);
	} catch (...) {
		cacheLock.unlock();
		throw ;
	}
	cacheLock.unlock();
}

Connection* CachedConnection::getBasicConnection()
{

	return inner->getBasicConnection();
}

Connection* CachedConnection::getInner()
{
	return inner;
}


}
