/**
 * @file cachedConnection.h
 *
 * @ingroup iopclib
 *
 * @author Petr Cermak
 */

#ifndef CACHEDCONNECTION_H_
#define CACHEDCONNECTION_H_

#include "iopcdb/connection.h"
#include "../cache/cache.h"
#include "../cache/cacheLock.h"
#include "../persIdentification.h"
#include "iopccommon/rwLock.h"
#include "cachedDatabase.h"

namespace iopc {


/**
 * @brief A @a Connection decorator which makes the @a Connection interface cache aware
 *
 * Each instance of @a CachedConnection has an associated @a Cache.
 * Behaviour of most of the @a Connection methods is modified to notify the cache on various events.
 * For example it tells the cache that it should write all cached local copies into database before commit is performed.
 * It also provides several proxy methods such as @a dbDelete(), @a cacheDelete(), @a makePersistent(), @a getCacheLock()
 * that invoke corresponding actions on the associated cache.
 *
 * @ingroup iopclib
 */
class CachedConnection : public Connection
{
private:
	Connection* inner;
	Cache* cache;
	CachedDatabase& database;
	RWLock cacheLock;

public:

	CachedConnection(Connection* connection, Cache* cache, CachedDatabase& database);

	/**
	 * @brief Empties and stops the associated cache
	 *
	 * @author Jan hadrava
	 */
	void stopCache();

	/**
	 * @brief Starts the stopped cache
	 *
	 * @author Jan hadrava
	 */
	void restartCache();

	/**
	 * @brief Deletes the inner (decorated) @a Connection instance and the associated @a Cache
	 */
	virtual ~CachedConnection();

	// Connection interface

	/**
	 * @brief Calls @a Cache::updateDirty() before performing @a Cursor::sql()
	 *
	 * @see Cursor::sql()
	 */
	virtual Cursor* sql(const std::string& name);

	/**
	 * @brief Executes an SQL statement while not notifying the associated cache
	 *
	 * @see sql()
	 */
	virtual Cursor* sqlNoCache(const std::string& name);

	virtual void returnCursor(Cursor* cur);

	virtual void closeAllCursors();

	/**
	 * @brief Calls @a Cache::updateDirty() before performing @a Cursor::sqlNonQuery()
	 *
	 * @see Cursor::sql()
	 */
	virtual void sqlNonQuery(const std::string& name);

	/**
	 * @brief Executes an SQL statement while not notifying the associated cache
	 *
	 * @see sqlNonQuery()
	 */
	virtual void sqlNonQueryNoCache(const std::string& name);

	/**
	 * @brief Calls @a Cache::updateDirty() before performing @a Cursor::commit()
	 *
	 * @see Cursor::commit()
	 */
	virtual void commit();

	/**
	 * @brief Calls @a Cache::removeAll() before performing @a Cursor::rollback()
	 *
	 * @see Cursor::rollback()
	 */
	virtual void rollback();

	/**
	 * @brief Calls @a Cache::updateDirty() before performing @a Cursor::savepoint()
	 *
	 * @see Cursor::savepoint()
	 */
	virtual void savepoint(const std::string& name);

	/**
	 * @brief Calls @a Cache::removeAll() before performing @a Cursor::rollbackToSavepoint()
	 *
	 * @see Cursor::rollbackToSavepoint()
	 */
	virtual void rollbackToSavepoint(const std::string& name);


	virtual void open();


	virtual void close();


	virtual void disconnect();

	virtual void abort();

	virtual void setAutoCommit(bool ac);

	virtual bool isOpened();

	virtual Connection& operator << (
	    const std::string& command
	);

	//Tests, if the Database connections are the same
	virtual bool operator == (const Connection& dbConn) const;
	//Tests, if the Database connections are not the same
	virtual bool operator != (const Connection& dbConn) const;

	inline Database& getDatabase() { return database; }

	inline Driver& getDriver() { return database.getDriver(); }


	/**
	 * @brief Invokes @a Cache::updateDirty() on the associated @a Cache
	 *
	 * Synchronizes the state of cache and DB by updating the DB.
	 */
	virtual inline void updateDirty() { cache->updateDirty(); }

	/**
	 * @brief Invokes @a Cache::removeAll() on the associated @a Cache
	 *
	 * Synchronizes the state of cache and DB by discarding all managed object
	 * (no modifications are written to the DB).
	 */
	virtual inline void removeAll() { cache->removeAll(); }

	inline ConnectionImpl* getImplementation() { return inner->getImplementation(); }

	virtual Connection* getBasicConnection();

	virtual Connection* getInner();

	/**
	 * @brief Transfers the referenced object ownership to the associated cache
	 *
	 * @param persId Identity of the object
	 * @param objectPtr The persistent object
	 */
	CacheLock& makePersistent(const PersIdentification& persId, DatabaseObject& objectPtr);

	/**
	 * @brief Locks the specified object in the associated cache
	 *
	 * @param persId Identity of the object to be locked
	 * @param rdonly Determines if a shared read-only lock should be placed on the object in the cache.
	 */
	CacheLock& getCacheLock(const PersIdentification& persId, bool rdonly = false);

	/**
	 * @brief Removes a persistent object identified by @a persId from database and from an associated cache.
	 */
	void dbDelete(const PersIdentification& persId);

	/**
	 * @brief Hints cache to drop its copy of the object identified by @a persId
	 */
	void cacheDelete(const PersIdentification& persId);

	/**
	 * @brief Returns the decorated connection
	 */
	virtual inline Connection* getInnerConnection() { return inner; }

	/**
	 * @brief Returns the associated cache
	 */
	inline Cache& getCache() { return *cache; }

};

}

#endif /*CACHEDCONNECTION_H_*/
