/**
 * @file mappingStatementsFeature.h
 *
 * @author Petr Cermak
 *
 * @ingroup iopclib
 */

#ifndef MAPPINGSTATEMENTSFEATURE_H_
#define MAPPINGSTATEMENTSFEATURE_H_

#include "iopcdb/dbFeature.h"
#include "iopccommon/metadataHolder.h"
#include "../types/dbType.h"
#include <vector>

namespace iopc {

class Type;

/**
 * @brief This feature declares methods shared by both O/R (vertical,  horizontal, filtered) and ADT mapping types
 *
 * @ingroup iopclib
 */
class MappingStatementsFeature : public DbFeature
{
private:
	static std::string featureName;
public:
	virtual const std::string& getNameInst() {
		return getName();
	}

	static const std::string& getName() {
		return featureName;
	}

	// interface

	/**
	 * @brief Returns a statement that selects a qualified @a Type name from the OID - Classname catalogue by an OID value
	 *
	 * @a OidObject metadata [db.table] and [db.column] should be used
	 */
	virtual std::string getSelectTypeStatement(const MetadataHolder& attributes = MetadataHolder::emptyMeta, bool useCache = true) =0;
	/**
	 * @brief Returns a statement that selects a timestamp (serialId) from the OID - Classname catalogue by an OID value
	 *
	 * @a OidObject metadata [db.table] and [db.column] should be used
	 */
	virtual std::string getSelectSerialIdStatement(const MetadataHolder& attributes = MetadataHolder::emptyMeta, bool useCache = true) =0;
	/**
	 * @brief Returns a statement that allocates and retrieves a new OID value
	 */
	virtual std::string getNextOidStatement(const MetadataHolder& attributes = MetadataHolder::emptyMeta, bool useCache = true) =0;
	/**
	 * @brief Returns a statement that allocates and retrieves a new timestamp (SerialId) value
	 */
	virtual std::string getNextSerialIdStatement(const MetadataHolder& attributes = MetadataHolder::emptyMeta, bool useCache = true) =0;
	/**
	 * @brief Returns a list of statements that are generated to the beginning of a DB CREATE script
	 */
	virtual std::vector<std::string> dbCreateScriptStart(const MetadataHolder& attributes = MetadataHolder::emptyMeta) =0;
	/**
	 * @brief Returns a list of statements that are generated to the end of a DB CREATE script
	 */
	virtual std::vector<std::string> dbCreateScriptEnd(const MetadataHolder& attributes = MetadataHolder::emptyMeta) =0;
	/**
	 * @brief Returns a list of statements that are generated to the beginning of a DB DROP script
	 */
	virtual std::vector<std::string> dbDropScriptStart(const MetadataHolder& attributes = MetadataHolder::emptyMeta) =0;
	/**
	 * @brief Returns a list of statements that are generated to the end of a DB DROP script
	 */
	virtual std::vector<std::string> dbDropScriptEnd(const MetadataHolder& attributes = MetadataHolder::emptyMeta) =0;
	/**
	 * @brief Returns a list of statements that create required structures to store instances of specified type to a database
	 *
	 * Following metadata should be taken into account:
	 * Type-level metadata:
	 * <ul>
	 *   <li>[db.table] - Table name</li>
	 *   <li>[db.type] - If ADT (object) mapping is used, denotes name of the ADT type to create.</li>
	 * </ul>
	 * Attribute-level metadata
	 * <ul>
	 *   <li>[db.column] - Column name</li>
	 *   <li>[db.type.notNull] - Column is rendered as NOT NULL</li>
	 *   <li>[db.primaryKey] - Column is part of the primary key</li>
	 * 	 <li>[db.index] - Name of a index to create</li>
	 * 	 <li>[db.index.unique] - Name of a unique index to create</li>
	 * </ul>
	 */
	virtual std::vector<std::string> getCreateClassTableScript(const DbType& dbType, const MetadataHolder& attributes = MetadataHolder::emptyMeta) =0;
	/**
	 * @brief Returns a list of statements that create a simple view for the specified type
	 *
	 * Simple view is a view that selects persistent objects only of the type they are associated to.
	 *
	 * Following metadata should be taken into account:
	 * Type-level metadata:
	 * <ul>
	 *   <li>[db.table] - Table name</li>
	 *   <li>[db.type] - If ADT (object) mapping is used, denotes name of the ADT type to use.</li>
	 *   <li>[db.view.sv] - Name of the Simple view</li>
	 * </ul>
	 * Attribute-level metadata
	 * <ul>
	 *   <li>[db.column] - Column name</li>
	 *   <li>[db.viewColumn] - View column name</li>
	 * </ul>
	 */
	virtual std::vector<std::string> getCreateUpDownViewsScript(const DbType& dbType, const MetadataHolder& attributes = MetadataHolder::emptyMeta) =0;
	/**
	 * @brief Returns a list of statements that create a plymorphic view for the specified type
	 *
	 * Polymorphic view is a view that selects persistent objects of the type they are associated to and of its descendants.
	 *
	 * Following metadata should be taken into account:
	 * Type-level metadata:
	 * <ul>
	 *   <li>[db.table] - Table name</li>
	 *   <li>[db.type] - If ADT (object) mapping is used, denotes name of the ADT type to use.</li>
	 *   <li>[db.view.pv] - Name of the Polymorphic view</li>
	 * </ul>
	 * Attribute-level metadata
	 * <ul>
	 *   <li>[db.column] - Column name</li>
	 *   <li>[db.viewColumn] - View column name</li>
	 * </ul>
	 */
	virtual std::vector<std::string> getCreateDownUpViewsScript(const DbType& dbType, const MetadataHolder& attributes = MetadataHolder::emptyMeta) =0;
	/**
	 * @brief Returns a list of statements that drop the structure created by @a getCreateClassTableScript
	 */
	virtual std::vector<std::string> getDropClassTableScript(const DbType& dbType, const MetadataHolder& attributes = MetadataHolder::emptyMeta) =0;
	/**
	 * @brief Returns a list of statements that drop the structures created by @a getCreateUpDownViewsScript
	 */
	virtual std::vector<std::string> getDropUpDownViewsScript(const DbType& dbType, const MetadataHolder& attributes = MetadataHolder::emptyMeta) =0;
	/**
	 * @brief Returns a list of statements that drop the structures created by @a getCreateDownUpViewsScript
	 */
	virtual std::vector<std::string> getDropDownUpViewsScript(const DbType& dbType, const MetadataHolder& attributes = MetadataHolder::emptyMeta) =0;
	/**
	 * @brief Returns a statement that is used to execute a query
	 *
	 * Following metadata should be taken into account:
	 * Query metadata (from the @a attributes parameter):
	 * <ul>
	 *   <li>[db.query.from] - From part of the generated select statement</li>
	 *	 <li>[db.query.useViewCols];bool - Determines whether the query select use [db.viewColumn] insetad of [db.column]</li>
	 *	 <li>[db.transient];bool - If the type is transient the query must select all columns from @a DbType::getAllPersistentAttributes()
	 *	   The order of the columns selected must correspond to the order of attributes in the AllPersistentAttributes list.
	 *	   If not transient, the selected columns must be obtained from @a DbType::getKeyAttributes(). In this case the column order must
	 *	   be kept too.</li>
	 *	 <li>[db.query.colPrefix] - Prefix rendered before each selected column name.</li>
	 *	 <li>[db.query.from] - The FROM part of the query. If missing, the query should select from [db.table] or [db.view.pv]
	 *	 depending on whether OID object is used (db.view.pv), or [db.query.useViewCols] is true (db.view.pv). [db.table] should be used
	 *	 for non-OID objects.</li>
	 *	 <li>[db.query.where] - The WHERE part of the query.</li>
	 *	 <li>[db.query.orderBy] - The ORDER BY part of the query.</li>
	 * </ul>
	 * Type-level metadata:
	 * <ul>
	 *   <li>[db.table] - Table name</li>
	 *   <li>[db.type] - If ADT (object) mapping is used, denotes name of the ADT type to use.</li>
	 *   <li>[db.view.pv] - Name of the Polymorphic view</li>
	 * </ul>
	 * Attribute-level metadata
	 * <ul>
	 *   <li>[db.column] - Column name</li>
	 *   <li>[db.viewColumn] - View column name</li>
	 * </ul>
	 */
	virtual std::string getQueryStatement(const DbType& dbType, const MetadataHolder& attributes) =0;
};


}


#endif /*MAPPINGSTATEMENTSFEATURE_H_*/
