/**
 * @file dbPtr.h
 *
 * @author Jan Hadrava
 * @author Petr Cermak
 *
 * @ingroup iopclib
 */

#ifndef DBPTR_H_
#define DBPTR_H_

#include "persIdentification.h"
#include "iopccommon/exceptionBase.h"
#include "cachePtr.h"
#include "constCachePtr.h"
#include "dbPtrBase.h"
#include "iopcmeta/typeDesc.h"
#include "databaseObject.h"
//#include "oidObject.h"

namespace iopc {

template<typename T> class CachePtr;
//template<typename T> class ConstCachePtr;

/**
 * @brief A database pointer class
 *
 * Database pointer is an abstraction of a persistent object reference. Database pointer may point to a persistent
 * object in several states:
 * <ul>
 *  <li>Transient instances which are present only in memory
 *  (and do not have a persistent representation yet), the owner of these instances is the pointer.
 *  State of the pointer is DbPtrBase::TRANSIENT</li>
 *  <li>In-memory instances owned by a cache (which may or may not have a database representation.
 *  This represents in fact two states of the object). State of the pointer is DbPtrBase::PERSISTENT</li>
 *  <li>Persistent representation of the object. The pointer contains only the identity of the object.
 *  State of the pointer is DbPtrBase::PERSISTENT</li>
 * </ul>
 *
 * Last object state, the locked local copy, is implemented as an additional level of indirection
 * when dereferencing the database pointer. By dereferencing it using the * operator (or using the -> operator or
 * by calling the @a getCachePtr() or @a getConstCachePtr method), user receives an instance
 * of a @e cache @e pointer (@a CachePtr<T> or @a ConstCachePtr<T>).
 *
 * Database pointer can be used in the following way:
 * @code
 * // Create a new empty Person persistent class instance
 * // The class is required to have an implitit or an explicit non-parametric constructor
 * DbPtr<Person> person;
 * // Assign values to its attributes
 * person->name = "Mary Major";
 * person->age = 60;
 * // Transfer the ownership of the object to a cache associated with the connection conn.
 * person.bePersistent(conn);
 * // Now the object data may be present either in the cache, in the cache and in the database, or in the database only
 * // So to be able to access its attributes the cache may need to load it from the database again.
 * // To prevent this behavior each time we access the cache, the object can be locked in a @a CachePtr cache pointer
 * // in the following way:
 * CachePtr<Person> p = *person;   // Finds the object in cache and loads it from a database if needed
 * p->name = "Mary Major 2";       // No cache lookup is necessare as the instance is locked in the @a CachePtr
 * p->age = 70;
 * p.release();                    // Now the instance is unlocked and the cache is free to store it to the database again.
 * @endcode
 *
 * @ingroup iopclib
 */
template<typename T> class DbPtr: public DbPtrBase {

public:
	/**
	 * @brief Crates a transient instance of the template parameter type
	 */
	DbPtr(/*bool createInstance = true*/);

	/**
	 * @brief Creates a copy of an existing database pointer
	 *
	 * If @a src points to a transient instance, this action adds 1 to its reference counter.
	 * The instance can't be delegated to a cache unless the reference count drops back to 1 (only one
	 * referencing database pointer).
	 *
	 * @param src Source database pointer
	 */
	DbPtr(const DbPtr<T> &src);

	/**
	 * @brief Loads a database object from an open cursor
	 *
	 * Used by @a Result<T> to fetch transient query result.
	 * Loaded object has no identity and can't be modified and made persistent.
	 *
	 * @param cur An open cursor from which one row will be fetched.
	 */
	DbPtr(Cursor& cur);

	/**
	 * @brief Initializes the identity of this database pointer from an OID
	 *
	 * This constructor makes the DbPtr<T> point to an object of the specified OID.
	 * State of the pointer will be DbPtrBase::PERSISTENT and the object will be looked up
	 * in the cache and eventually loaded from a database when one of its attributes is accessed.
	 *
	 * @param conn A cached connection with an associated cache
	 * @param oid OID of the object this pointer will refer to
	 * @param loadRealType Determines whether polymorphism is used when loading the object instance
	 * If false, a T template parameter instance is created and loaded. If true, the actual type the
	 * oid represents is first looked up in the Oid-classname catalogue and according to it a persistent
	 * object instance is created. I.e. if OID 2 refers to a Student class and the database pointer is
	 * declared as DbPtr<Person>, a Student instance will be created and loaded. The instance can be
	 * then obtained via
	 * @code
	 * dynamic_cast<Student&>(*(getCachePtr().getObjectPtr()))
	 * @endcode
	 */
	DbPtr(CachedConnection* conn, const OidType& oid, bool loadRealType = true);

	/**
	 * @brief Initializes the identity of this database pointer from a set of key values
	 *
	 * This constructor makes the DbPtr<T> point to an object of the specified key values.
	 * State of the pointer will be DbPtrBase::PERSISTENT and the object will be looked up
	 * in the cache and eventually loaded from a database when one of its attributes is accessed.
	 *
	 * @param conn A cached connection with an associated cache
	 * @param keyVals Values of the object primary key
	 */
	DbPtr(CachedConnection* conn, const KeyValues& keyVals);


	//DbPtr(CachedConnection* conn);

	~DbPtr();

	DbPtr<T> &operator=(const DbPtr<T> &src);


	/**
	 * @brief Makes the referenced object persistent.
	 *
	 * Generates a new identity for the object and transfers the referenced object ownership
	 * to an associated cache
	 */
	CachePtr<T> bePersistent(CachedConnection* conn);


	/**
	 * @brief Dereferences the database pointer.
	 *
	 * @return An instance representing locked copy of referenced object.
	 */
	inline CachePtr<T> operator*() const;

	/**
	 * @brief Dereferences the database pointer.
	 *
	 * @returns An instance representing locked copy of referenced object.
	 */
	CachePtr<T> operator->() const;

	/**
	 * @brief Dereferences the database pointer.
	 *
	 * @returns An instance representing locked copy of referenced object.
	 */
	CachePtr<T> getCachePtr() const;

	/**
	 * @brief Dereferences the database pointer for reading.
	 *
	 * @returns An instance representing copy locked for reading (shared lock).
	 */
	ConstCachePtr<T> getConstCachePtr() const;

private:
/*	CachePtr<T> getNewCachePtr();
	ConstCachePtr<T> getNewConstCachePtr();*/

	CountedPtr<T> objectPtr;



protected:
	CountedPtr<T> getObjectPtr() const { return objectPtr; }

/*	inline DbPtr(T& objectPtr)
		throw ();*/

	virtual bool isObjectPtrNull() const { return objectPtr.isNull(); }
	virtual bool isObjectPtrUnique() const { return objectPtr.unique(); }
	virtual DatabaseObject& getObject() const { return dynamic_cast<DatabaseObject&>(*objectPtr); }
	virtual void setObjectPtrNull(bool deleteObject = true) { objectPtr.allowDelete(deleteObject); objectPtr = CountedPtr<T>(); }
};


template<typename T> DbPtr<T>::DbPtr(/*bool createInstance*/)
: DbPtrBase(TypeDesc<T>::getType()/*, createInstance*/), objectPtr(CountedPtr<T>(dynamic_cast<T*>(TypeDesc<T>::getType().newObjectInstance())))
{
	callOnIopcCreate(objectPtr.get());
	//objectPtr->onIopcCreate();
	//setNewInstance(dynamic_cast<DatabaseObject*>(TypeDesc<T>::getType().newObjectInstance()));
}

template<typename T> DbPtr<T>::DbPtr(Cursor& cur)
: DbPtrBase(TypeDesc<T>::getType()), objectPtr()
{
	DatabaseObject* obj = getPersId().load(cur);
	if (obj == NULL) {
		return; // isValid() will return false
	} else {
		objectPtr = CountedPtr<T>(dynamic_cast<T*>(obj));
	}
}

template<typename T> DbPtr<T>::DbPtr(const DbPtr<T>& src)
: DbPtrBase(src), objectPtr(src.objectPtr)
{
	//src.objectPtr;
	/*if (src.getState() == PERSISTENT) {
		objectPtr = CountedPtr<T>();
	} else {
		objectPtr = src.objectPtr;
	}*/
}

template<typename T> DbPtr<T>::DbPtr(CachedConnection* conn, const OidType& oid, bool loadRealType)
: DbPtrBase(TypeDesc<T>::getType(), conn, oid, loadRealType), objectPtr()
{
}

template<typename T> DbPtr<T>::DbPtr(CachedConnection* conn, const KeyValues& keyVals)
: DbPtrBase(TypeDesc<T>::getType(), conn, keyVals), objectPtr()
{
}

/*template<typename T> inline DbPtr<T>::DbPtr(T &objectPtr)
	throw ()
: DbPtrBase(*T::getType(), false)
{
	ASSERT(persId.keyValues == NULL && conn == NULL);
	this->objectPtr = &objectPtr;
}*/

/*template<typename T> DbPtr<T>::DbPtr(Connection &conn, const QueRefProto &query)
	throw (PoliteExceptionNoMemory)
: DbPtrBase(*T::StPrototype(), true)
{
	 *this = ++Result<T>(conn, query);
}*/

template<typename T> DbPtr<T>::~DbPtr()
{
	//deleteObject();
}

template<typename T> DbPtr<T> &DbPtr<T>::operator=(const DbPtr<T>& src)
{
	if (this == &src)
		return *this;

	objectPtr = src.objectPtr;
	/*if (src.getState() == PERSISTENT) {
		objectPtr = CountedPtr<T>();
	} else {
		objectPtr = src.objectPtr;
	}*/

	replaceWith(src);

	return *this;
}

/*template<typename T> inline DbPtr<T> *DbPtr<T>::transientClone() const
	throw ()
{
	Object *obj = this->getConstCachePtr().getObjectPtr()->clone();
	obj->setTransient();
	obj->setDirty(false);
	return new DbPtr<T>(*(T *)obj);
}*/

template<typename T> CachePtr<T> DbPtr<T>::bePersistent(CachedConnection* conn)
{
	CacheLock& cLock = DbPtrBase::bePersistent(conn);
	propagateConnectionToAttributes(cLock.getObject());
	return CachePtr<T>(cLock);
}

template<typename T> CachePtr<T> DbPtr<T>::operator*() const
{
	return getCachePtr();
}

template<typename T> CachePtr<T> DbPtr<T>::operator->() const
{
	return getCachePtr();
}

template<typename T> CachePtr<T> DbPtr<T>::getCachePtr() const
{
	switch (getState()) {
		case PERSISTENT:
		{
				CachePtr<T> ret = CachePtr<T>(getConnection(), getPersId());
				DbPtrBase::propagateConnectionToAttributes(*ret.getObjectPtr().get());
				return ret;
			break;
		}
		case TRANSIENT:
		{
				if (getObjectPtr().isNull()) {
					IOPC_LOG_THROW(IopcInvalidStateException("Object was deleted or does not exist"));
				}
				return CachePtr<T>(getObjectPtr());
			break;
		}
		default:
			IOPC_UNREACHABLE;
	}
	IOPC_UNREACHABLE;
}

template<typename T> ConstCachePtr<T> DbPtr<T>::getConstCachePtr() const
{
	switch (getState()) {
		case PERSISTENT:
		{
				ConstCachePtr<T> ret = ConstCachePtr<T>(getConnection(), getPersId());
				DbPtrBase::propagateConnectionToAttributes(*ret.getObjectPtr().get());
				return ret;
			break;
		}
		case TRANSIENT:
		{
				if (getObjectPtr().isNull()) {
					IOPC_LOG_THROW(IopcInvalidStateException("Object was deleted or does not exist"));
				}
				return ConstCachePtr<T>(getObjectPtr());
			break;
		}
		default:
			IOPC_UNREACHABLE;
	}
	IOPC_UNREACHABLE;
}

}

#endif /*DBPTR_H_*/
