/**
 * @file iopc.h
 *
 * @author Petr Cermak
 *
 * @ingroup iopclib
 * @defgroup iopclib iopclib
 */


#ifndef IOPC_H_
#define IOPC_H_

#include "iopccommon/trace.h"

namespace iopc {

class MetadataInitializer;

/**
 * @brief @c iopclib library/configuration initialization class
 *
 * When using the @c iopclib configuration, the library must be initialized
 * by calling the @a Iopc::init() method and terminated by calling
 * the @a Iopc::shutdown() method.
 *
 * @c iopccommon and iopcmeta and iopcdb libraries are initialized and terminated implicitly
 *
 * @ingroup iopclib
 */
class Iopc {

	public:
		/**
		 * @brief Initializes the @c iopclib library/configuration
		 *
		 * @param traceMaskTTY @a LogWriter mask used for console output
		 * @param traceMask @a LogWriter mask used for log file output
		 * @param logFileName Name of the IOPC log file
		 */
		static void init(
				int traceMaskTTY = LogWriter::defaultTraceMaskTTY,
				int traceMask = LogWriter::defaultTraceMask,
				const std::string& logFileName = LogWriter::defaultFileName);

		/**
		 * @brief Initializes the @c iopclib library/configuration. @a MetadataInitializer can be specified.
		 *
		 * @a metadataInitializer is called during the library initialization.
		 * After that the @a MetadataInitializer instance is deallocated.
		 *
		 * @param metadataInitializer The @a MetadataInitializer
		 * @param traceMaskTTY @a LogWriter mask used for console output
		 * @param traceMask @a LogWriter mask used for log file output
		 * @param logFileName Name of the IOPC log file
		 */
		static void init(
				MetadataInitializer* metadataInitializer,
				int traceMaskTTY = LogWriter::defaultTraceMaskTTY,
				int traceMask = LogWriter::defaultTraceMask,
				const std::string& logFileName = LogWriter::defaultFileName);

		/**
		 * @brief Terminates the @c iopclib library/configuration and all dependencies
		 */
		static void shutdown();

};


}

#endif /*IOPC_H_*/
