/**
 * @file oidObject.h
 *
 * @author Petr Cermak
 *
 * @ingroup iopclib
 */

#ifndef OIDOBJECT_H_
#define OIDOBJECT_H_

#include "databaseObject.h"

namespace iopc {

typedef EULong OidType;
typedef unsigned long OidTypeInner;
typedef EULong SerialIdType;


/**
 * @brief Provides an OID identity and all O/R mapping features to its descendants
 *
 * @a OidObject guarantees an identity in form of an internally generated OID
 * to all of its descendants. Unique OID is assigned to any instance of an OidObject descendant
 * at its creation time (more preciesly - when the @a DbPtrBase::bePersistent() method is invoked)
 * and remains unchanged during its lifetime. @a OidObject descendants are able to use all types of database mappings.
 * Oid objects depend on database structures generated by the IOPC 2 library. These structures
 * include simple or polymorphic views or an Oid - classname catalogue.
 * The catalogue is actually a regular mapping table associated with this class. Each OID object
 * in the database is represented by a row in the catalogue regardless of the mapping type used.
 * The catalogue table has three columns - OID, CLASSNAME (qualified name of the class this row - OID - belongs to)
 * and SERIALID (timestamp used by the cache layer).
 *
 * @ingroup iopclib
 */
class DLL_EXTERNAL OidObject : public DatabaseObject {

private:
	static void iopcInit(iopc::Type& t) {
		if (!t.hasMeta("db.table")) {
			t["db.table"] = "OidObject";
		}
		//t.getAttribute("oid")["db.sequence"] = "OIDSEQ";
		const Attribute& aOid = t.getAttribute("oid");
		if (!aOid.hasMeta("db.column")) {
			aOid["db.column"] = "OID";
		}
		t.getAttribute("oid")["db.primaryKey"].setBoolValue(true);
		if (!aOid.hasMeta("db.viewColumn")) {
			aOid["db.viewColumn"] = "OID";
		}
		const Attribute& aClassName = t.getAttribute("className");
		if (!aClassName.hasMeta("db.column")) {
			aClassName["db.column"] = "CLASSNAME";
		}
		if (!aClassName.hasMeta("db.type.length")) {
			aClassName["db.type.length"] = 1000;
		}
		if (!aClassName.hasMeta("db.viewColumn")) {
			aClassName["db.viewColumn"] = "CLASSNAME";
		}
		if (!aClassName.hasMeta("db.index")) {
			aClassName["db.index"] = "idxcn";
		}
		const Attribute& aSerialId = t.getAttribute("serialId");
		if (!aSerialId.hasMeta("db.column")) {
			aSerialId["db.column"] = "SERIALID";
		}
		if (!aSerialId.hasMeta("db.viewColumn")) {
			aSerialId["db.viewColumn"] = "SERIALID";
		}
	}

protected:
	void onIopcCreate();

	virtual void onCreate() {};

public:
	/**
	 * @brief The OID that identifies this object
	 *
	 * It is initialied as soon as the DbPtrBase::bePersistent() method is called or
	 * when the object is loaded from the database.
	 */
	OidType oid;

	/**
	 * @brief className of this class descendant
	 *
	 * It is initialied as soon as the database pointer DbPtr<T> is created or
	 * when the object is loaded from the database.
	 */
	EString className;

	/**
	 * @brief A timestamp used by the cache layer
	 */
	SerialIdType serialId;

	inline virtual bool isOidObject() {
		return true;
	};

	virtual ~OidObject() {};


	friend class DbPtrBase; // onIopcCreate();
	friend class TypeDesc<OidObject>;
};

// TypeDesc for OidType

}

// generate TypeDesc
META_BLOCK1(iopc::OidObject)
META_PARENT1(iopc::DatabaseObject)
META_ATTR1(oid, OidType, Attribute::VISIBILITY_PUBLIC)
META_ATTR1(className, EString, Attribute::VISIBILITY_PUBLIC)
META_ATTR1(serialId, SerialIdType, Attribute::VISIBILITY_PUBLIC)
META_BLOCK2(iopc::OidObject)
META_ATTR2(iopc::OidObject, oid)
META_ATTR2(iopc::OidObject, className)
META_ATTR2(iopc::OidObject, serialId)
META_BLOCK3(iopc::OidObject, OidObject, iopc)
META_INIT(iopc::OidObject)
META_BLOCK4


/*
namespace iopc {
template <> class TypeDesc< iopc::OidObject> : public Type {
public:
	TypeDesc<  iopc::OidObject>() {
		parentsTypeInfo.push_back(&typeid(DatabaseObject));
		attributes.insert(std::make_pair("oid", Attribute("oid", TypeDesc< OidType>::getType(), Attribute::VISIBILITY_PUBLIC, *this)));
		attributes.insert(std::make_pair("className", Attribute("className", TypeDesc< EString>::getType(), Attribute::VISIBILITY_PUBLIC, *this)));
		attributes.insert(std::make_pair("serialId", Attribute("serialId", TypeDesc< SerialIdType>::getType(), Attribute::VISIBILITY_PUBLIC, *this)));
	}
	inline virtual void fillAttributeValues(const Object& o, std::map<std::string, AttributeValue>& out) const {
		for(std::map<std::string, Attribute>::const_iterator it = attributes.begin(); it != attributes.end(); it++) {
			std::string attrName = (*it).first;

			if (attrName == "oid") { std::cout << std::string("attr: ") << attrName << " " << reinterpret_cast<uint> ((void*)&dynamic_cast<const iopc::OidObject&>(o).oid) << std::endl; out.insert(std::make_pair(attrName, AttributeValue((*it).second, &dynamic_cast<const iopc::OidObject&>(o).oid))); continue; }
			if (attrName == "className") { std::cout << std::string("attr: ") << attrName << " " << reinterpret_cast<uint> ((void*)&dynamic_cast<const iopc::OidObject&>(o).className) << std::endl; out.insert(std::make_pair(attrName, AttributeValue((*it).second, &dynamic_cast<const iopc::OidObject&>(o).className))); continue; }
			if (attrName == "serialId") { std::cout << std::string("attr: ") << attrName << " " << reinterpret_cast<uint> ((void*)&dynamic_cast<const iopc::OidObject&>(o).serialId) << std::endl; out.insert(std::make_pair(attrName, AttributeValue((*it).second, &dynamic_cast<const iopc::OidObject&>(o).serialId))); continue; }
		}
	}

	virtual ~TypeDesc< iopc::OidObject>() {};
	static inline const TypeDesc< iopc::OidObject>& getType() {static TypeDesc< iopc::OidObject> instance; return instance; };
	virtual void* newInstance() const { return new iopc::OidObject(); };
	virtual Object* newObjectInstance() const { return dynamic_cast<Object*>((iopc::OidObject*) new iopc::OidObject()); }
	inline virtual DataTypeClassEnum getDataTypeClass() const { return IOPC_TYPECLASS_COMPLEX; }
	inline virtual DataTypeEnum getDataType() const { return IOPC_TYPE_COMPLEX; };
	virtual int getSizeOf(const void* object) const { return sizeof(iopc::OidObject); };
	virtual const std::string getName() const{return "OidObject";};
	virtual const std::string getNamespace() const {return "iopc";};
	inline virtual bool isBuiltIn() const { return false; };
	//inline virtual const std::map<std::string, Attribute>& getAttributes() const { return attributes; };
	virtual void iopcInit() { iopc::OidObject::iopcInit(*this); }
};
}*/






#endif /*OIDOBJECT_H_*/
