/**
 * @file ObjectMappingStrategy.cpp
 *
 * @author Petr Cermak
 *
 * @ingroup iopclib
 */


#include "objectMappingStrategy.h"
#include "dbType.h"
#include "dbTypeManager.h"
#include "iopcdb/connection.h"
#include "../db/objectStatementsFeature.h"
#include "iopcdb/cursor.h"
#include "iopcdb/driver.h"
#include "iopccommon/exceptionBase.h"

using namespace iopc;
using namespace std;

void ObjectMappingStrategy::init()
{
	if (!getDbType().isOidManaged()) {
		IOPC_LOG_THROW(IopcMetadataException("db.mapping.type", getDbType().getType().getQualifiedName(), "Object must be descendant of the OidObject class"));
	}

	root = true;
	for (Type::ParentsIterator it = getType().getParents().begin(); it != getType().getParents().end(); it++)
	{
		if ((*it)->isTypeOf<OidObject>()) {
			DbType& dbType = DbTypeManager::getDbTypeWithInit(**it);
			if (dbType.getMappingType() == DbType::MAPPING_OBJECT) {
				root = false;
				ObjectMappingStrategy& parentStrategy = (ObjectMappingStrategy&) dbType.getMappingStrategy();
				roots.insert(parentStrategy.getRootTypes().begin(), parentStrategy.getRootTypes().end());
			}
		}
	}
	if (root) {
		roots.insert(&getDbType());
	}
}

void ObjectMappingStrategy::postInit()
{
	Type::ChildrenList children = getType().getChildren();
	for (Type::ChildrenIterator it = children.begin(); it != children.end(); it++) {
		DbType& dbType = DbTypeManager::getDbType(**it);
		if (dbType.getMappingType() != DbType::MAPPING_OBJECT) {
			IOPC_LOG_THROW(IopcMetadataException("db.mapping.type", dbType.getType().getQualifiedName(), "Descendants of 'object-mapping' types must have also object mapping"));
		}
	}

	for (DbType::PersistentAttributesIterator it = getDbType().getAllPersistentAttributes().begin(); it != getDbType().getAllPersistentAttributes().end(); it++) {
		const Attribute* attr = *it;
		const DbType& owner = DbTypeManager::getDbType(attr->getOwner());
		if (owner.getMappingType() != DbType::MAPPING_OBJECT) continue;
		if (*attr == getOidColumn()) continue;

		attributes.push_back(attr);
	}

	for (DbType::PersistentAttributesIterator it = getDbType().getPersistentAttributes().begin(); it != getDbType().getPersistentAttributes().end(); it++) {
		if (**it == getOidColumn()) continue;
		persistentAttributes.push_back(*it);
	}
}


void ObjectMappingStrategy::loadOne(Connection& conn, DatabaseObject* obj, const KeyValues& keyVals, bool wait, bool lock)
{
	MetadataHolder meta;
	if (lock) {
		meta.setFeature("db.select.lock");
	}
	if (!wait) {
		meta.setFeature("db.select.nowait");
	}
	const string selectStatement = conn.getDriver().getFeature<ObjectStatementsFeature>().getSelectFromClassTableStatement(*this, meta);

	Cursor* cur = conn.sql(selectStatement);

	mapKeyAttributes(cur, 1, keyVals);
	int i = 1;
	for (PersistentAttributesIterator it = getMappedAttributes().begin(); it != getMappedAttributes().end(); it++) {
		const AttributeValue& attrVal = obj->getAttributeValue((*it)->getName());
		cur->addOutParam(i, attrVal.getValue(), attrVal.getAttribute().getType(), (MetadataHolder&) attrVal.getAttribute());
		i++;
	}

	cur->execute();
	if (!cur->fetchNext()) {
		IOPC_LOG_THROW(IopcNoRowsFetchedException("No row was fetched during the O/R mapping"));
	}
	cur->close();
	conn.returnCursor(cur);
}

OidObject* ObjectMappingStrategy::viewLoadAll(Connection& conn, const OidType& oid, bool wait, bool lock)
{
	const string selectStatement = conn.getDriver().getFeature<ObjectStatementsFeature>().getSelectFromClassViewStatement(*this);
	//const string selectStatement = "";


	OidObject* obj = (OidObject*)getType().newObjectInstance();

	try {
		//LogWriter::logSql(selectStatement);
		Cursor* cur = conn.sql(selectStatement);
		cur->addInParam(1, &oid, TypeDesc<OidType>::getType());
		int i = 1;
		for (DbType::PersistentAttributesIterator it = getDbType().getAllPersistentAttributes().begin(); it != getDbType().getAllPersistentAttributes().end(); it++) {
			const Attribute& attr = *(*it);
			const AttributeValue& attrVal = obj->getAttributeValue(attr.getName());
			//cur->addOutParam(i, )
			cur->addOutParam(i, attrVal.getValue(), attrVal.getAttribute().getType(), (MetadataHolder&) attrVal.getAttribute());
			i++;
		}

		cur->execute();
		//cur->preFetchBind(1, &oid, sizeof(OidType), TypeDesc<OidType>::getType());
		if (!cur->fetchNext()) {
			IOPC_LOG_THROW(IopcNoRowsFetchedException("No row was fetched during the O/R mapping"));
		}
		cur->close();
		conn.returnCursor(cur);
		return obj;

	} catch (...) {
		delete obj;
		throw ;
	}
}

void ObjectMappingStrategy::updateOne(Connection& conn, DatabaseObject& obj)
{
	Cursor* cur;

	const string updateStatement = conn.getDriver().getFeature<ObjectStatementsFeature>().getUpdateStatement(*this);
	cur = conn.sql(updateStatement);

	int i = 1;
	for (PersistentAttributesIterator it = getMappedAttributes().begin(); it != getMappedAttributes().end(); it++) {
		const AttributeValue& attrVal = obj.getAttributeValue((*it)->getName());
		cur->addInParam(i, attrVal.getValue(), attrVal.getAttribute().getType(), (MetadataHolder&) attrVal.getAttribute());
		i++;
	}

	mapKeyAttributes(cur, i, obj);

	cur->execute();
	cur->close();
	conn.returnCursor(cur);
}


void ObjectMappingStrategy::deleteOne(Connection& conn, const OidType& oid)
{
	IOPC_FUNC_INVOKED
	const string deleteStatement = conn.getDriver().getFeature<ObjectStatementsFeature>().getDeleteStatement(*this);
	Cursor* cur = conn.sql(deleteStatement);
	mapKeyAttributes(cur, 1, oid);
	cur->execute();
	cur->close();
	conn.returnCursor(cur);
	IOPC_FUNC_ENDED;
}

void ObjectMappingStrategy::deleteOne(Connection& conn, const KeyValues& keyVals)
{
	IOPC_FUNC_INVOKED
	const string deleteStatement = conn.getDriver().getFeature<ObjectStatementsFeature>().getDeleteStatement(*this);
	Cursor* cur = conn.sql(deleteStatement);
	mapKeyAttributes(cur, 1, keyVals);
	cur->execute();
	cur->close();
	conn.returnCursor(cur);
	IOPC_FUNC_ENDED;
}

void ObjectMappingStrategy::insertOne(Connection& conn, DatabaseObject& obj)
{
	IOPC_FUNC_INVOKED;
	const string insertStatement = conn.getDriver().getFeature<ObjectStatementsFeature>().getInsertStatement(*this);
	IOPC_DEBUGLOG(insertStatement);
	Cursor* cur = conn.sql(insertStatement);

	// first goes the oid
	mapKeyAttributes(cur, 1, obj);
	int i = getDbType().getKeyAttributes().size() + 1;
	for (PersistentAttributesIterator it = getMappedAttributes().begin(); it != getMappedAttributes().end(); it++) {
		const AttributeValue& attrVal = obj.getAttributeValue((*it)->getName());
		cur->addInParam(i, attrVal.getValue(), attrVal.getAttribute().getType(), (MetadataHolder&) attrVal.getAttribute());
		i++;
	}

	cur->execute();
	cur->close();
	conn.returnCursor(cur);

	IOPC_FUNC_ENDED;
}
