/**
 * @file ObjectMappingStrategy.h
 *
 * @author Petr Cermak
 *
 * @ingroup iopclib
 */

#ifndef OBJECTMAPPINGSTRATEGY_H_
#define OBJECTMAPPINGSTRATEGY_H_

#include "mappingStrategy.h"
#include "iopcmeta/type.h"
#include "dbType.h"
#include <set>
#include "dbTypeManager.h"

namespace iopc {

/**
 * @brief Implements attribute-level part of the O/R mapping algorithm for the ADT (object) mapping types
 *
 * @ingroup iopclib
 */
class ObjectMappingStrategy : public MappingStrategy
{
private:
	DbType* dbType;
public:
	ObjectMappingStrategy(DbType* dbType) : dbType(dbType) {};
	virtual void init();
	virtual void postInit();
	virtual void loadOne(Connection& conn, DatabaseObject* obj, const KeyValues& keyVals, bool wait, bool lock);
	virtual OidObject* viewLoadAll(Connection& conn, const OidType& oid, bool wait, bool lock);
	virtual void updateOne(Connection& conn, DatabaseObject& obj);
	virtual void deleteOne(Connection& conn, const OidType& oid);
	virtual void deleteOne(Connection& conn, const KeyValues& keyVals);
	virtual void insertOne(Connection& conn, DatabaseObject& obj);
	inline virtual DbType& getDbType() const { return *dbType; }
	inline virtual const Type& getType() const { return dbType->getType(); }
public:
	typedef std::set<const DbType*> RootTypesList;
	typedef RootTypesList::const_iterator RootTypesIterator;

	typedef std::vector<const Attribute*> PersistentAttributesList;
	typedef PersistentAttributesList::const_iterator PersistentAttributesIterator;
private:
	PersistentAttributesList attributes;
	PersistentAttributesList persistentAttributes;
	RootTypesList roots;
	bool root;

public:
	/**
	 * @brief Determines whether the associated type is root of the ADT type hierarchy
	 */
	inline bool isRoot() const { return root; }

	/**
	 * @brief Returns a list of types that represent roots of the ADT hierarchy the associated type is part of.
	 */
	inline const RootTypesList& getRootTypes() const { return roots; }

	/**
	 * @brief Returns first root from the @a getRootTypes() list.
	 */
	inline const ObjectMappingStrategy& getFirstRoot() const { return (*roots.begin())->getObjectMappingStrategy(); }
	/**
	 * @brief Returns a list of all persistent attributes from the associated type and its ADT-mapped ancestors.
	 */
	inline const PersistentAttributesList& getMappedAttributes() const { return attributes; }
	/**
	 * @ brief Same as DbType::getPersistentAttributes(), but it doesn't contain the oid column,  even for the OidObject
	 */
	inline const PersistentAttributesList& getPersistentAttributes() const { return persistentAttributes; }

	/**
	 * @brief Returns reference to the OID attribute
	 */
	inline const Attribute& getOidColumn() const { return getType().getAttributeAll("oid"); }

	/**
	 * @brief Returns @a ObjectMappingStrategy of the first parrent of the associated type
	 *
	 * If the parent does not use ADT (object) mapping, the behavioru of this method is undefined
	 */
	inline ObjectMappingStrategy& getFirstParentOMStrategy() const { return DbTypeManager::getDbTypeWithInit(getType().getFirstParent()).getObjectMappingStrategy(); }
};

}
#endif /*OBJECTMAPPINGSTRATEGY_H_*/
