/**
 * @file scriptsGenerator.h
 *
 * @author Petr Cermak
 *
 * @ingroup iopclib
 */

#ifndef SCRIPTSGENERATOR_H_
#define SCRIPTSGENERATOR_H_

#include <string>
#include <vector>

namespace iopc {

class Connection;
class Driver;
class Type;

/**
 * @brief Generates SQL CREATE and DROP scripts that create or delete database schema required for O/R mapping.
 *
 * The generator first creates a list of topologically ordered persistent classes according to their inheritance dependencies
 * and then calls the @a MappingStatementsFeature for each of them in order to generate the scripts.
 *
 * Following example shows how to prepare required database schema
 * @code
 * vector<string> script;
 * script = ScriptsGenerator::getDbCreateScript(conn->getDriver());
 * for(vector<string>::const_iterator it = script.begin(); it != script.end(); it++) {
 *	conn->sqlNonQuery(*it);
 * }
 * @endcode
 *
 * @ingroup iopclib
 */
class ScriptsGenerator {
public:
	/**
	 * @brief Generates SQL CREATE script that creates required database schema
	 */
	static std::vector<std::string> getDbCreateScript(const Driver& driver);

	/**
	 * @brief Generates SQL DROP script that drops the schema generated by @a getDbCreateScript()
	 *
	 * Note that if the class model has been changed between create nad drop script generation,
	 * the drop script may not be reliable and should be revised.
	 */
	static std::vector<std::string> getDbDropScript(const Driver& driver);
};



}


#endif /*SCRIPTSGENERATOR_H_*/
