/**
 * @file attributeValue.h
 *
 * @ingroup iopcmeta
 *
 * @author Petr Cermak
 */

#ifndef ATTRIBUTEVALUE_H_
#define ATTRIBUTEVALUE_H_

#include "iopccommon/defs.h"
#include "attribute.h"
#include <string>

namespace iopc {

class Type;

/**
 * @brief Represents an attribute value
 *
 * @a AttributeValue contains reference to a corresponding @a Attribute
 * and a void pointer to the attribute value. Data type of the value can be determined
 * by calling
 * @code
 * const Type& t = attrVal.getAttribute().getType();
 * @endcode
 *
 * Value of an attribute from existing @a Object descendant instance can be obtained
 * for example by calling
 * @code
 * const AttributeValue& attrVal = obj.getAttributeValue("attr_name");
 * @endcode
 *
 * Users are not supposed to create instances of this class
 *
 * @ingroup iopcmeta
 */
class DLL_EXTERNAL AttributeValue {
public:
	AttributeValue(const Attribute& attr, const void* value) : attribute(&attr), value(const_cast<void*>(value)) {};
	AttributeValue(const AttributeValue& src) : attribute(src.attribute), value(src.value) {};

	/**
	 * @brief Returns correspoinding @a Attribute
	 */
	inline const Attribute& getAttribute() const { return *attribute; };

	/**
	 * @brief Returns a void pointer to the attribute value
	 */
	inline void* getValue() const { return value; };

	inline AttributeValue& operator=(const AttributeValue& src)  { attribute = src.attribute; value = src.value; return *this; }
	inline bool operator!=(const AttributeValue& src) const { return this != &src; }
	inline bool operator==(const AttributeValue& src) const { return this == &src; }
	inline bool operator<(const AttributeValue& src) const { return this < &src; }
private:
	const Attribute* attribute;
	void* value;
};

}



#endif /*ATTRIBUTEVALUE_H_*/
