/**
 * @file typeDesc.h
 *
 * @ingroup iopcmeta
 *
 * @brief Contains definition of the @a TypeDesc<T> type description template class and
 * its specialisations for selected C++ basic numeric types and STL strings @a std::string and @a std::wstring
 *
 * @author Petr Cermak
 */

#ifndef TYPEDESC_H_
#define TYPEDESC_H_

#include "iopccommon/defs.h"
#include "type.h"
#include "typeManager.h"
#include <string>
#include "object.h"


// jinak budou tyhle retezce v kazdem modulu, ktery tohle .h includuje.


namespace iopc {

/**
 * @brief Type Description template class. Implements the @a Type interface.
 *
 * Specialisations of this class are either defined in the IOPC 2 library or
 * generated by @c iopcsp using the macros from @a iopcmeta/metaSupport.h
 *
 * @ingroup iopcmeta
 */
template<typename T> class DLL_EXTERNAL TypeDesc : public Type {
public:
	virtual DataTypeEnum getDataType() const;
	virtual const std::string getName() const;
	virtual const std::string getNamespace() const;
	inline virtual void* newInstance() const { return new T; };
	inline virtual Object* newObjectInstance() const { return dynamic_cast<Object*>((T*) new T);};
	inline virtual void deleteInstance(void* instance) const { delete (T*) instance; };
	static const TypeDesc<T>& getType();
	virtual int getSizeOf(const void* object) const;
	virtual DataTypeClassEnum getDataTypeClass() const;
	virtual void fillAttributeValues(const Object& o, std::map<std::string, AttributeValue>& out) const;
	virtual bool isBuiltIn() const;
private:
	TypeDesc();
	static TypeDesc<T> instance;
};

#define SIMPLE_TYPE_DEFINITION(name, type, ns, typeName, dataType, dataTypeClass) \
template <> class DLL_EXTERNAL TypeDesc<type> : public Type {	\
public:	\
	inline virtual DataTypeEnum getDataType() const { return dataType; }	\
	inline virtual const std::string getName() const { return name; }	\
	inline virtual const std::string getNamespace() const { return ns; } \
	inline virtual void* newInstance() const { return new type; } \
	inline virtual Object* newObjectInstance() const { return NULL; } \
	inline virtual int getSizeOf(const void*) const { return sizeof(type); }	\
	static inline const TypeDesc<type>& getType() { return instance; }	\
	inline virtual DataTypeClassEnum getDataTypeClass() const { return dataTypeClass; } \
	inline virtual void fillAttributeValues(const Object& o, std::map<std::string, AttributeValue>& out) const {} \
	inline virtual bool isBuiltIn() const { return true; } \
	inline virtual void deleteInstance(void* instance) const { delete (type*) instance; } \
private:	\
	TypeDesc() { TypeManager::registerType(this, typeName, &typeid(type)); }		\
	static TypeDesc<type> instance;	\
}

SIMPLE_TYPE_DEFINITION("bool", bool, "", "::bool", IOPC_TYPE_BOOL, IOPC_TYPECLASS_SIMPLE);
SIMPLE_TYPE_DEFINITION("char", char, "", "::char", IOPC_TYPE_CHAR, IOPC_TYPECLASS_SIMPLE);
SIMPLE_TYPE_DEFINITION("unsigned char", unsigned char, "", "::unsigned char", IOPC_TYPE_UCHAR, IOPC_TYPECLASS_SIMPLE);
SIMPLE_TYPE_DEFINITION("signed char", signed char, "", "::signed char", IOPC_TYPE_SCHAR, IOPC_TYPECLASS_SIMPLE);
SIMPLE_TYPE_DEFINITION("short", short, "", "::short", IOPC_TYPE_SHORT, IOPC_TYPECLASS_SIMPLE);
SIMPLE_TYPE_DEFINITION("unsigned short", unsigned short, "", "::unsigned short", IOPC_TYPE_USHORT, IOPC_TYPECLASS_SIMPLE);
SIMPLE_TYPE_DEFINITION("int", int, "", "::int", IOPC_TYPE_INT, IOPC_TYPECLASS_SIMPLE);
SIMPLE_TYPE_DEFINITION("unsigned int", unsigned int, "", "::unsigned int", IOPC_TYPE_UINT, IOPC_TYPECLASS_SIMPLE);
SIMPLE_TYPE_DEFINITION("long", long, "", "::long", IOPC_TYPE_LONG, IOPC_TYPECLASS_SIMPLE);
SIMPLE_TYPE_DEFINITION("unsigned long", unsigned long, "", "::unsigned long", IOPC_TYPE_ULONG, IOPC_TYPECLASS_SIMPLE);
SIMPLE_TYPE_DEFINITION("float", float, "", "::float", IOPC_TYPE_FLOAT, IOPC_TYPECLASS_SIMPLE);
SIMPLE_TYPE_DEFINITION("double", double, "", "::double", IOPC_TYPE_DOUBLE, IOPC_TYPECLASS_SIMPLE);
SIMPLE_TYPE_DEFINITION("long double", long double, "", "::long double", IOPC_TYPE_LONGDOUBLE, IOPC_TYPECLASS_SIMPLE);
SIMPLE_TYPE_DEFINITION("wchar_t", wchar_t, "", "::wchar_t", IOPC_TYPE_WCHART, IOPC_TYPECLASS_SIMPLE);

SIMPLE_TYPE_DEFINITION("Object", Object, "iopc", "iopc::Object", IOPC_TYPE_COMPLEX, IOPC_TYPECLASS_COMPLEX);

template <> class DLL_EXTERNAL TypeDesc<std::string> : public Type {
public:
	inline virtual DataTypeEnum getDataType() const { return IOPC_TYPE_STRING; }
	inline virtual const std::string getName() const { return "string"; }
	inline virtual const std::string getNamespace() const { return "std"; }
	inline virtual void* newInstance() const { return new std::string(); }
	inline virtual Object* newObjectInstance() const { return NULL; }
	inline virtual int getSizeOf(const void* val) const { return ((std::string*) val)->length() + 1; }
	static inline const TypeDesc<std::string>& getType() { return instance; }
	inline virtual DataTypeClassEnum getDataTypeClass() const { return IOPC_TYPECLASS_STRING; }
	inline virtual void fillAttributeValues(const Object& o, std::map<std::string, AttributeValue>& out) const {}
	inline virtual bool isBuiltIn() const { return true; }
	inline virtual void deleteInstance(void* instance) const { delete (std::string*) instance; }
private:
	TypeDesc() { TypeManager::registerType(this, "std::string", &typeid(std::string)); }
	static TypeDesc<std::string> instance;
};

template <> class DLL_EXTERNAL TypeDesc<std::wstring> : public Type {
public:
	inline virtual DataTypeEnum getDataType() const { return IOPC_TYPE_WSTRING; }
	inline virtual const std::string getName() const { return "wstring"; }
	inline virtual const std::string getNamespace() const { return "std"; }
	inline virtual void* newInstance() const { return new std::wstring(); }
	inline virtual Object* newObjectInstance() const { return NULL; }
	inline virtual int getSizeOf(const void* val) const { return (((std::wstring*) val)->length() + 1) * sizeof(wchar_t); }
	static inline const TypeDesc<std::wstring>& getType() { return instance; }
	inline virtual DataTypeClassEnum getDataTypeClass() const { return IOPC_TYPECLASS_STRING; }
	inline virtual void fillAttributeValues(const Object& o, std::map<std::string, AttributeValue>& out) const {}
	inline virtual bool isBuiltIn() const { return true; }
	inline virtual void deleteInstance(void* instance) const { delete (std::wstring*) instance; }
private:
	TypeDesc() { TypeManager::registerType(this, "std::wstring", &typeid(std::wstring)); }
	static TypeDesc<std::wstring> instance;
};

}
#endif /*TYPEDESC_H_*/
